C
C***********************************************************************
C*                                                                     *
C*============================                                         *
C*    SUBROUTINE BASES( FXN )                                          *
C*============================                                         *
C*((Function))                                                         *
C*    Subroutine performs N-dimensional Monte Carlo integration        *
C*    for four vector generation of simulated events                   *
C*                                                                     *
C*       IFLAG = 0 ; First Trial of Defining Grid                      *
C*       IFLAG = 1 ; First Trial of Data Accumulation                  *
C*       IFLAG = 2 ; Second Trial of Defining Grid                     *
C*       IFLAG = 3 ; Second Trial of Data Accumulation                 *
C*                                                                     *
C*    Coded   by S.Kawabata    July 1980 at DESY, Hamburg              *
C*    Last update by S.Kawabata Oct 1985 at KEK.                       *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BASES( FXN )
C
      IMPLICIT REAL*8 (A-H,O-Z)
      EXTERNAL FXN
      REAL*4   SETIM,UTIME,TIM1,TIM2,RTIM
      PARAMETER (MXDIM = 25, NDMX = 50, LENG = 17000)
      COMMON /BASE0/ SETIM,UTIME,IFLAG,NPRINT,IBASES
      COMMON /BASE1/ XL(MXDIM),XU(MXDIM),NDIM,NGDIM,
     .               IG(MXDIM),NCALL
      COMMON /BASE2/ ACC1,ACC2,ITMX1,ITMX2
      COMMON /BASE3/ SI,SI2,SWGT,SCHI,SCALLS,ATACC,NSU,IT,WGT
      COMMON /BASE4/ XI(NDMX,MXDIM),DX(MXDIM),DXD(LENG),DXP(LENG),
     .               ND,NG,NPG,MA(MXDIM)
      PARAMETER (ITM = 100)
      REAL*4  TIME,EFF,WRONG,RESLT,ACSTD,TRSLT,TSTD,PCNT
      COMMON /BASE5/ ITRAT(ITM),TIME(ITM),EFF(ITM),WRONG(ITM),
     .       RESLT(ITM),ACSTD(ITM),TRSLT(ITM),TSTD(ITM),PCNT(ITM)
      COMMON /BASE6/ D(NDMX,MXDIM)
      COMMON /BSRSLT/AVGI,SD,CHI2A,STIME,ITF
      COMMON /LOOP0/ LOOPC,IRCODE
C
      DIMENSION X(MXDIM),KG(MXDIM),IA(MXDIM)
      REAL*4    BIPER(3)
      CHARACTER*50 ICH
      CHARACTER*1  BLNK,STAR
      DATA BLNK,STAR / ' ','*'/
      DATA BIPER/ 1.,2.,5./
      DATA  ONE/ 1.0/, ZERO/0.0/, N0/0/, N1/1/, N2/2/, HUNDRT/100.0/
      DATA  NDM0 / 0/, NDM1/ 0/, NDM2/ 0/
C
      IBASES  = 1
      MPRINT  = ABS(NPRINT)
C
C===============================================================
C                 Initialization Part
C===============================================================
C
C  -------------------------------------------------------------
C           Define the number of hypercubes
C -------------------------------------------------------------
      IF( IFLAG .LE. N0 ) THEN
         CALL BSINIT
         CALL BSTIME( TIM0, 1 )
C
      ELSEIF( NDM2 .NE. 1 ) THEN
         CALL BSTIME( TIM0, 1 )
         NDM2  = 1
      ENDIF
 
C  -----------------------------------------------------------------
 
      XND     = ND
      NSP     = NG**NGDIM
      XJAC    = 1.0
      DO  5 I = 1, NDIM
         XJAC = XJAC*DX(I)
    5 CONTINUE
      CALLS   = NSP*NPG
      DXG     = 1.0/NG
      DV2G    = DXG**(2*NGDIM)/NPG/NPG/(NPG-1)
      DXG     = DXG*XND
 
      NEND    = N0
      ITOVER  = N0
      ATACC   = ZERO
 
      IF(IFLAG .EQ. N0 .OR. IFLAG .EQ. N1 ) THEN
         DO 10 J  = N1,NSP
           DXD(J) = ZERO
           DXP(J) = ZERO
   10    CONTINUE
         ISTEP   = IFLAG
         IT1   = N1
         SI    = ZERO
         SI2   = ZERO
         SWGT  = ZERO
         SCHI  = ZERO
C        -----------
         CALL BHRSET
C        -----------
         NSU     = N0
         SCALLS= ZERO
      ELSE
          IF(    IFLAG .EQ. 2 ) THEN
                 ISTEP  = N0
          ELSEIF(IFLAG .EQ. 3 ) THEN
                 ISTEP  = N1
          ELSE
                 WRITE(6,9000) IFLAG
 9000            FORMAT(1X,'BASES Error; Illegal FLAG =',I3 )
 
C                *****************
                       STOP
C                *****************
          ENDIF
 
          IT1   = IT + 1
      ENDIF
 
      ITMX   = ITMX1
      ACC    = ACC1*0.01
      IF( ISTEP .EQ. N1 ) THEN
         ITMX = ITMX2
         ACC  = ACC2*0.01
      ENDIF
C  -----------------------------------------------------------------
 
      IF( NDIM .NE. NDM0 ) THEN
         MCALL  = CALLS
         WRITE(6,9050)  NDIM,NGDIM,ITMX1,ITMX2,ACC1,ACC2,MCALL,NG,ND
 9050    FORMAT(//20X,
     .          '******************************************',
     .     /20X,'*    INPUT PARAMETERS FOR BASES          *',
     .     /20X,'*                                        *',
     .     /20X,'* NDIM  =',I9,   '  NGDIM =',I9,   '     *',
     .     /20X,'* ITMX1 =',I9,   '  ITMX2 =',I9,   '     *',
     .     /20X,'* ACC1  = ',G9.3, ' ACC2  = ',G9.3, '    *',
     .     /20X,'* NCALL =',I9,   '                       *',
     .     /20X,'* NG    =',I9,   '  ND    =',I9,   '     *')
         DO 100 I = 1,NDIM
            WRITE(6,9100) I,XL(I),I,XU(I),I,IG(I)
 9100       FORMAT(20X,
     .          '* XL(',I3,' )= ',G15.6,      '              *',
     .     /20X,'* XU(',I3,' )= ',G15.6,      ' IG(',I3,' )= ',I2,' *')
  100    CONTINUE
         WRITE(6,9150)
 9150    FORMAT(20X,
     .          '*                                        *',
     .     /20X,'******************************************')
         NDM0  = NDIM
      ENDIF
C
C====================================================================
C               Main Integration Loop
C====================================================================
C
      NEGFLG     = 0
      DO 500  IT = IT1,ITMX
 
            SCALLS  = SCALLS + CALLS
 
            NGOOD = N0
            NEGTIV= N0
 
            CALL BSTIME( TIM1, 1 )
 
            TI    = ZERO
            TSI   = TI
 
            IF( ISTEP .EQ. N0 ) THEN
                DO 230 J= N1,NDIM
                DO 230 I=1,ND
                   D(I,J)= TI
  230           CONTINUE
            ENDIF
C---------------------------------------------------------------------
C            Accumulate Events  for all cubes
C---------------------------------------------------------------------
            DO 380 NCB = 1,NSP
 
               NP       = NCB - 1
               IF( NGDIM .GT. 1 ) THEN
                   DO 250 J = 1,NGDIM-1
                      NUM   = MOD(NP,MA(J+1))
                      KG(J) = NUM/MA(J) + 1
  250              CONTINUE
               ENDIF
 
               KG(NGDIM)     = NP/MA(NGDIM) + 1
C
C -------------------------------------------------------------------
C
               FB    = ZERO
               F2B   = ZERO
C
               DO 350 NTY = 1,NPG
C
                  WGT   = XJAC
                  DO 300 J= N1,NDIM
                     IF( J .LE. NGDIM ) THEN
                         XN    = (KG(J)-DRN(DUMY))*DXG+ONE
                     ELSE
                         XN    = ND*DRN(DUMY)+ONE
                     ENDIF
                     IA(J) = XN
                     IAJ   = IA(J)
                     IF(IAJ .EQ. N1) THEN
                         XO    = XI(IAJ,J)
                         RC    = (XN-IA(J))*XO
                     ELSE
                         XO    = XI(IAJ,J)-XI(IAJ-N1,J)
                         RC    = XI(IAJ-N1,J)+(XN-IAJ)*XO
                     ENDIF
                     X(J)  = XL(J)+RC*DX(J)
                     WGT   = WGT*XO*XND
  300             CONTINUE
C      ----------------------------------------------------
                  FXG   = FXN(X)
                  F     = FXG*WGT
C
                  IF( FXG .NE. 0.0 ) THEN
                     NGOOD = NGOOD + N1
                     IF( ISTEP .EQ. N1 ) THEN
                        DXD(NCB) = DXD(NCB) + F
                        IF( F .GT. DXP(NCB) ) DXP(NCB) = F
                     ENDIF
                     IF( FXG .LT. ZERO) THEN
                         NEGTIV= NEGTIV+ N1
                         IF( NEGFLG .EQ. N0 ) THEN
                             WRITE(6,9200) IT
 9200                        FORMAT(1X,
     .                        '******* WARNING FROM BASES ********',
     .                    /1X,'*  Negative FUNCTION at IT =',I3,3X,'*',
     .                    /1X,'***********************************')
                             NEGFLG  = 1
                         ENDIF
                     ENDIF
                  ENDIF
 
 
                  F2    = F*F
                  FB    = FB + F
                  F2B   = F2B + F2
 
                  IF( ISTEP .EQ. N0 ) THEN
                      DO 320  J = N1,NDIM
                         D(IA(J),J)= D(IA(J),J)+F2
  320                 CONTINUE
                  ENDIF
C         PRINT *,'F,F2  = ',F,F2
C
  350          CONTINUE
C     -----------------------------------------------------------
               F2B   = DSQRT(F2B*NPG)
               F2S   = (F2B-FB)*(F2B+FB)
               TI    = TI+FB
               TSI   = TSI + F2S
C       PRINT *,'NSC,F2B,FB = ',NSC,F2B,FB
C       PRINT *,'F2S,TI,TSI = ',F2S,TI,TSI
C
  380       CONTINUE
C
C---------------------------------------------------------------------
C           Compute Result of this Iteration
C---------------------------------------------------------------------
C
            TI    = TI/CALLS
            TSI   = TSI*DV2G
            TI2   = TI*TI
C
            CALL BHSAVE
C
C
            IF( NGOOD .LE. 10 ) THEN
              WRITE(6,9250)
 9250         FORMAT(1X,'******** FATAL ERROR IN BASES **************',
     .        /1X,'There are no enough good points in this iteration.',
     .        /1X,'This job was forced stop.')
 
C             *****************
                    STOP
C             *****************
 
            ENDIF
C
            WGT   = ONE/TSI
            SI    = SI+TI*WGT
            SWGT  = SWGT+WGT
            SCHI  = SCHI+TI2*WGT
            AVGI  = SI/SWGT
            CHI2A = ZERO
            IF(IT .GT. N1 ) CHI2A = (SCHI - SI*AVGI)/(IT-.999)
            SD    = DSQRT(ONE/SWGT)
 
            CALL BSTIME( TIM2, 1 )
 
            TSI   = DSQRT(TSI)
            ITX         = MOD( IT, ITM)
            IF( ITX .EQ. 0 ) ITX = ITM
            ITRAT(ITX)  = IT
            TIME( ITX)  = TIM2
            EFF ( ITX)  = NGOOD/CALLS*HUNDRT
            WRONG(ITX)  = NEGTIV/CALLS*HUNDRT
            RESLT(ITX)  = AVGI
            ACSTD(ITX)  = SD
            TRSLT(ITX)  = TI
            TACC        = ABS(TSI/TI*HUNDRT)
            TSTD( ITX)  = TACC
            PCNT( ITX)  = ABS(SD/AVGI*HUNDRT)
C
C  -------------------------------------------------------------------
C           Check remaining CPU time for the next iteration
C  -------------------------------------------------------------------
            STIME = TIM2 - TIM0
            UTIME = (TIM2-TIM1)*1.1
            RTIM  = SETIM - TIM2
            IF(RTIM .LE. UTIME) ITOVER = N1
C  -------------------------------------------------------------------
C           Check cumulative accuracy and  the iteration number.
C  -------------------------------------------------------------------
            SDAV  = SD/AVGI
            IF((ABS(SDAV) .LE. ACC)) NEND = N1
 
            IF( NEND .EQ. N1 ) GO TO 600
 
            IF(ISTEP .LE. N0 ) THEN
C  ------------------------------------------------------------------
C          Test Temporary Accuracy  whether it becoms stable.
C  ------------------------------------------------------------------
 
                AACC = ABS(TACC-ATACC)/TACC
                IF( AACC .LE. 0.01) NSU = NSU + N1
                IF( AACC .GT. 0.01) NSU = N0
                ATACC = TACC
                IF( NSU .GT. 3 ) THEN
                    NEND = N1
                    GO TO 600
                ENDIF
 
C  ------------------------------------------------------------------
C        Smoothing the Distribution D(I,J) and refine the grids
C  ------------------------------------------------------------------
C               -----------
                CALL BSGDEF
C               -----------
            ENDIF
C --------------------------------------------------------------------
C         Test the Flag NEND ; If NEND = (1/2) = (Time out/ Converge)
C---------------------------------------------------------------------
C
            IF( ITOVER .EQ. N1 ) GO TO 600
C
  500    CONTINUE
         IT    = IT - N1
         NEND  = N1
C
C  ===================================================================
C                   Termination of BASES
C  ===================================================================
C  -------------------------------------------------------------------
C                   Print Convergence behavior
C  -------------------------------------------------------------------
C
  600    IF( MPRINT. GE. 4 .OR.
     .     ( MPRINT .GE. 3 .AND. ISTEP .EQ. N1)) THEN
 
            PCNTMX = PCNT(1)
            IF( ITRAT(1) .EQ. 1 ) THEN
                NDEV   = 1
            ELSE
                NDEV   = 2
                ITFN   = ITM
                ITMN   = 10000
                DO 610 I = 1,ITM
                   IF( PCNT(I) .GT. PCNTMX) PCNTMX = PCNT(I)
                   IF( ITRAT(I) .LT. ITMN ) THEN
                        ITST = I
                        ITMN = ITRAT(I)
                   ENDIF
  610           CONTINUE
                IF( ITST .EQ. 1 ) NDEV = 1
            ENDIF
 
            DO 620 JJ = -1,3
               ORDER  = 10.0**JJ
               DO 620 KK =  1, 3
                  PERMAX  = BIPER(KK)*ORDER
                  DPER    = PERMAX/50.
                  IF(PERMAX .GE. PCNTMX ) GO TO 622
  620       CONTINUE
  622       WRITE(6,9600) PERMAX
 9600       FORMAT(1H1,/1H ,
     .                1X,' IT',4X,'Time',2X,'Eff.',2X,'R_Neg',
     .                6X,'RESULT',6X,'ST-DEV',6X,'T-RES',4X,'T-ACC',
     .                5X,'Acc',10X,'Maximum of Dist =',G11.3)
            WRITE(6,9610)
 9610       FORMAT(1X,12('----------'),'---------+')
C
  625       IF( NDEV .EQ. 1 ) THEN
                ITST = 1
                ITFN = ITX
            ENDIF
 
            DO 650 I = ITST, ITFN
               IPER  = PCNT(I)/DPER
               IF(IPER .GT. 50) IPER = 50
               DO 630  K = N1, 50
                  IF(K .LE. IPER) THEN
                     ICH(K:K)= STAR
                  ELSE
                     ICH(K:K)= BLNK
                  ENDIF
  630          CONTINUE
               WRITE(6,9630) ITRAT(I),TIME(I),EFF(I),WRONG(I),RESLT(I),
     .              ACSTD(I),TRSLT(I),TSTD(I),PCNT(I),ICH
 9630          FORMAT(1X,I4,F8.2,F6.1,F7.2,1P,D14.6,
     .                  2D11.3, 0P,2F8.3,' I',A50,'I')
  650       CONTINUE
            NDEV  = NDEV - 1
            IF( NDEV .GT. 0 ) GO TO 625
 
            WRITE(6,9610)
         ENDIF
C
C  -------------------------------------------------------------------
C                      Print out the result
C  -------------------------------------------------------------------
C
         IF( (MPRINT .GT. 0 .AND. MPRINT .LE. 2) .AND.
     .       ISTEP .EQ. N1 .AND. NEND .EQ. N1 ) THEN
            IF(  NDIM .NE. NDM1 ) THEN
               WRITE(6,9650)
 9650          FORMAT(/1H1,//1H ,
     .             15X,'RESULT OF INTEGRATION',
     .            /10X,'+----+-----------+-----------+',
     .                 '-----------+-----------+-----+',
     .            /10X,'|LOOP|  ESTIMATE |  ST-DEV.  |',
     .                 ' CHI**2/IT |  CPU TIME |  IT |',
     .            /10X,'+----+-----------+-----------+',
     .                 '-----------+-----------+-----+')
               NDM1  = NDIM
            ENDIF
            WRITE(6,9660) LOOPC,AVGI,SD,CHI2A,STIME,IT
 9660       FORMAT(10X,'|',I3,' |',4(G10.4,' |'),I4,' |',
     .            /10X,'+----+-----------+-----------+',
     .                 '-----------+-----------+-----+')
         ENDIF
C
         IF( MPRINT .GE. 5 .OR.
     .     ( MPRINT .GE. 3 .AND. ISTEP.EQ. N1 ) .OR.
     .     ( MPRINT .EQ. 2 .AND. ISTEP.EQ. N1 .AND. NEND .EQ. N1 )) THEN
C            -----------
             CALL BHPLOT
C            -----------
         ENDIF
C
C  -----------------------------------------------------------------
C                     Time out ?
C  -----------------------------------------------------------------
C
         IF(ITOVER .EQ. N1 ) THEN
            IF(ISTEP .GE. N0 ) THEN
               IF(ISTEP .EQ. N0) THEN
                  IFLAG = N2
C                 -----------
                  CALL BSGDEF
C                 -----------
               ENDIF
               IF(ISTEP .EQ. N1) IFLAG = 3
            ENDIF
C
         ENDIF
C
C -------------------------------------------------------------------
C                      END OF BASES ( Convergence )
C -------------------------------------------------------------------
C
         IF(NEND .EQ. N1 ) THEN
            IF(ISTEP .EQ. N0) THEN
               IFLAG   = N1
            ELSE
               IFLAG   = N0
               ITF     = IT
            ENDIF
         ENDIF
C
         RETURN
         END
C
C***********************************************************************
C*======================================                               *
C*    SUBROUTINE BHINIT( MAXLH, MAXDH )                                *
C*=====================================                                *
C*((Purpose))                                                          *
C*   Initialization program for  histograms and scatter plots.         *
C*   This program is called by USERIN.                                 *
C*((Arguements))                                                       *
C*   MAXLH  : Maximum number of the linear histograms                  *
C*   MAXDH  : Maximum number of the scatter plots                      *
C*                                                                     *
C* ((Author))                                                          *
C*   S.Kawabata    June '90 at KEK                                     *
C***********************************************************************
C
      SUBROUTINE BHINIT( MAXLH, MAXDH )
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
 
C    NW                     : Total number of words of used buffer
C
C    NHIST                  : Number of Histograms
C    NSCAT                  : Number of Scat_Plots
C
C **********************
C     Hashing Table
C **********************
C
C        XHASH(1,i)      : Number of histograms for the i-th class
C        XHASH(2,i) = K  : Serial number of histograms
C                     |
C              MAPL(1,K) = ID  : Histogram ID
C              MAPL(2,K) = IP1 : the 1st pointer to the K-th buffer
C              MAPL(3,K) = IP2 : the 2nd pointer to the K-th buffer
C              MAPL(4,K) = IP3 : the 3rd pointer to the K-th buffer
C
C
C ************************
C     Histogram buffer
C ************************
C
C    IP1  = NW + 1
C           NW = NW + 281    : Updated NW
C       BUFF( IP1 )          = Xmin
C       BUFF( IP1 + 1 )      = Xmax
C       IBUF( IP1 + 2 )      = No. of bins
C       BUFF( IP1 + 3 )      = Bin width
C    IP2  = IP1 + 4
C       IBUF(   IP2       )
C          => IBUF( +  51 )  = No. of sampling points
C       BUFF(   IP2 +  52)
C          => BUFF( + 103 )  = Sum of Fi for the current IT
C       BUFF(   IP2 + 104)
C          => BUFF( + 155 )  = Sum of Fi**2 for the current IT
C       BUFF(   IP2 + 156)
C          => BUFF( + 207 )  = Sum of Fi for total
C       BUFF(   IP2 + 208)
C          => BUFF( + 259 )  = Sum of Fi**2 for total
C    IP3  = IP1 + 264
C       IBUF( IP3 )          = Tag for spring
C       IBUF( IP3   +  1 )
C          => IBUF( + 16 )   = Title of this histogram
C
C ************************
C     Scat_Plot buffer
C ************************
C
C IP1   = NW + 1
C         NW  = NW + 2527
C       BUFF( IP1 )          = Xmin
C       BUFF( IP1 + 1 )      = Xmax
C       IBUF( IP1 + 2 )      = No. of bins for X
C       BUFF( IP1 + 3 )      = Bin width for X
C       BUFF( IP1 + 4 )      = Ymin
C       BUFF( IP1 + 5 )      = Ymax
C       IBUF( IP1 + 6 )      = No. of bins for Y
C       BUFF( IP1 + 7 )      = Bin width for Y
C IP2   = IP1 + 8
C       BUFF(   IP2       )  = No. of sampling points
C       BUFF(   IP2 +   1 )
C          => BUFF( +2500 )  = Sum of Fi
C IP3   = IP1 + 2509
C       IBUF( IP3 )          = X-Tag for spring
C       IBUF( IP3   +  1 )   = Y-Tag for spring
C       IBUF( IP3   +  2 )
C          => IBUF( + 17 )   = Title of this histogram
C
      COMMON/XHCNTL/ LOCK
 
        IF( LOCK .NE. 0 ) RETURN
 
         IF( MAXLH .GT. ILH ) THEN
             WRITE(6,9000)
 9000        FORMAT(1X,'Too many histogram buffer are required ||')
             STOP
         ELSEIF( MAXD .GT. IDH ) THEN
             WRITE(6,9100)
 9100        FORMAT(1X,'Too many Scat_Plot buffer are required ||')
         ENDIF
 
         MAXL   = MAXLH + 1
         MAXD   = MAXDH
         NW     = 0
 
         DO 50 I = 1, 13
           XHASH(1,I) = 0
           DHASH(1,I) = 0
   50    CONTINUE
         NHIST    = 0
         NSCAT    = 0
         DO 100 I = 1, ILH
           MAPL(1,I)= 0
  100    CONTINUE
         DO 200 I = 1, IDH
           MAPD(1,I)= 0
  200    CONTINUE
C
      RETURN
      END
C***********************************************************************
C*=======================                                              *
C*    SUBROUTINE BHPLOT                                                *
C*=======================                                              *
C*((Purpose))                                                          *
C*    Interface routine to print histograms and scatter plots.         *
C*    Routines XHPLOT and DHPLOT are called to print them.             *
C*((Author))                                                           *
C*    S.Kawabata  June '90  at KEK                                     *
C***********************************************************************
C
      SUBROUTINE BHPLOT
C
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
 
      CALL XHCHCK
 
      IF( NHIST .LE. 0 ) THEN
         WRITE(6,9000)
 9000    FORMAT(1X,'No Histogram')
      ELSE
         DO 500 J = 1, NHIST
            IFBASE(J) = 1
C
            CALL XHPLOT( 0, J )
C
  500    CONTINUE
      ENDIF
C
      CALL DHPLOT
C
      RETURN
      END
C***********************************************************************
C*=======================                                              *
C*    SUBROUTINE BHRSET                                                *
C*=======================                                              *
C*((Purpose))                                                          *
C*    To reset contents of histograms and scatter plots.               *
C*((Author))                                                           *
C*    S.Kawabata  June '90 at KEK                                      *
C***********************************************************************
C
      SUBROUTINE BHRSET
C
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
C
         DO 200 J    = 1, NHIST
           IP2       = MAPL(3,J)
           DO 100 I  = IP2,IP2+259
             IBUF(I) = 0
  100      CONTINUE
           IFBASE(J) = 0
  200    CONTINUE
C
         DO 500  J   = 1, NSCAT
           IP2       = MAPD(3,J)
           DO 400  I = IP2,IP2+2500
             IBUF(I) = 0.0
  400      CONTINUE
  500    CONTINUE
C
      RETURN
      END
C***********************************************************************
C*=======================                                              *
C*    SUBROUTINE BHSAVE                                                *
C*=======================                                              *
C*((Purpose))                                                          *
C*    To reset contents of histograms and scatter plots.               *
C*((Author))                                                           *
C*    S.Kawabata  June '90 at KEK                                      *
C***********************************************************************
C
      SUBROUTINE BHSAVE
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
 
      DO 200 J = 1, NHIST
         IP2   = MAPL(3,J)
         NC    = IBUF( MAPL(2,J)+2 ) + 1
         IB1   = IP2 + 52
         IB2   = IB1 + 52
         DO 100 I = 0,NC
            I1    = I + IB1
            I2    = I1 + 104
            BUFF(I2)  = BUFF(I2) + BUFF(I1)
            BUFF(I1)  = 0.0
            I1    = I + IB2
            I2    = I1 + 104
            BUFF(I2)  = BUFF(I2) + BUFF(I1)
            BUFF(I1)  = 0.0
  100    CONTINUE
  200 CONTINUE
C
      RETURN
      END
C***********************************************************************
C*                                                                     *
C*========================                                             *
C*    SUBROUTINE BSGDEF                                                *
C*========================                                             *
C*((Function))                                                         *
C*    Refine the grid sizes                                            *
C*                                                                     *
C*    Coded   by S.Kawabata    Aug. 1984 at Nagoya Univ.               *
C*    Last update              Oct. 1985 at KEK                        *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BSGDEF
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXDIM = 25, NDMX = 50, LENG = 17000)
      COMMON /BASE1/ XL(MXDIM),XU(MXDIM),NDIM,NGDIM,
     .               IG(MXDIM),NCALL
      COMMON /BASE4/ XI(NDMX,MXDIM),DX(MXDIM),DXD(LENG),DXP(LENG),
     .               ND,NG,NPG,MA(MXDIM)
      COMMON /BASE6/ D(NDMX,MXDIM)
 
      DIMENSION  XIN(NDMX),R(NDMX),DT(MXDIM)
      DATA ALPH / 1.5 /,ONE/1.0/,ZERO/0.0/,N0/0/,N1/1/
C
C======= SMOOTHING THE FUNCTION D(I,J)
C
        CLOGE   = 1.0D0/LOG(10.0D0)
 
        NDM     = ND-1
        DO 780 J= N1,NDIM
         IF( IG(J) .EQ. 1 ) THEN
          XO    = D(1,J)
          XN    = D(2,J)
          D(1,J)= (XO+XN)/2.
          DT(J) = D(1,J)
          DO 710 I=2,NDM
            D(I,J)= XO+XN
            XO    = XN
            XN    = D(I+N1,J)
            D(I,J)= (D(I,J)+XN)/3.
            DT(J) = DT(J)+D(I,J)
  710     CONTINUE
          D(ND,J) = 0.5*(XN+XO)
          DT(J)   = DT(J)+D(ND,J)
C
C=========== REDEFINE THE GRID
C
 
          DTLOG   = LOG(DT(J))
          DT10    = CLOGE*DTLOG
          RC    = ZERO
          DO 730 I= N1,ND
            R(I)  = ZERO
            IF(D(I,J) .GT. ZERO) THEN
               DILOG = LOG(D(I,J))
               IF( DT10 - CLOGE*DILOG  .LE. 70.0 ) THEN
                   XO    = DT(J)/D(I,J)
                   R(I)  = ((XO-ONE)/(XO*(DTLOG-DILOG)))**ALPH
               ELSE
                   XO    = DT(J)/D(I,J)
                   R(I)  = (DTLOG-DILOG)**(-ALPH)
               ENDIF
            ENDIF
            RC    = RC+R(I)
  730     CONTINUE
          RC    = RC/ND
          K     = N0
          XN    = N0
          DR    = XN
          I     = K
  740  K     = K + N1
          DR    = DR+R(K)
          XO    = XN
          XN    = XI(K,J)
  750 IF(RC.GT.DR)GO TO 740
          I     = I + N1
          DR    = DR-RC
          XIN(I)= XN-(XN-XO)*DR/R(K)
      IF(I.LT.NDM)GO TO 750
          DO 760 I= N1,NDM
            XI(I,J)= XIN(I)
  760     CONTINUE
          XI(ND,J)= ONE
         ENDIF
  780   CONTINUE
C
      RETURN
      END
C***********************************************************************
C*                                                                     *
C*========================                                             *
C*    SUBROUTINE BSINIT                                                *
C*========================                                             *
C*((Function))                                                         *
C*     Initialization of Bases progam                                  *
C*     This is called only when IFLAG=0.                               *
C*     ( IFLAG = 0 ; First Trial of Defining Grid step )               *
C*                                                                     *
C*    Changed by S.Kawabata    Aug. 1984 at Nagoya Univ.               *
C*    Last update              Oct. 1985 at KEK                        *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BSINIT
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXDIM = 25, NDMX = 50, LENG = 17000)
      COMMON /BASE1/ XL(MXDIM),XU(MXDIM),NDIM,NGDIM,
     .               IG(MXDIM),NCALL
      COMMON /BASE3/ SI,SI2,SWGT,SCHI,SCALLS,ATACC,NSU,IT
      COMMON /BASE4/ XI(NDMX,MXDIM),DX(MXDIM),DXD(LENG),DXP(LENG),
     .               ND,NG,NPG,MA(MXDIM)
 
      DIMENSION  XIN(NDMX)
      DATA  ONE/ 1.0/
C
C---------------------------------------------------------------
C           Set initial seeds of random number generator
C---------------------------------------------------------------
      ISEED = 12345
C
      CALL DRNSET( ISEED )
C
C---------------------------------------------------------------
C           Define the number of grids and sub-regions
C---------------------------------------------------------------
C==> Determine NG : Number of grids
          NG    = (NCALL/2.)**(1./NGDIM)
         IF(NG .GT. 25) NG  = 25
  100    IF(NG .LT.  2) NG  =  1
         IF(NG**NGDIM .GT. LENG) THEN
            NG  = NG - 1
            GO TO 100
         ENDIF
C
C==> Determine ND : Number of sub-regions
          M     = NDMX/NG
          ND    = M*NG
C
C==> Determine NPG: Number of sampling points per subhypercube
          NSP   = NG**NGDIM
          NPG   = NCALL/NSP
 
          XI(1,1)= ONE
          MA(1)  = 1
          DX(1)  = XU(1)-XL(1)
 
          IF( NDIM .GT. 1 ) THEN
              DO 130 J = 2,NDIM
                 XI(1,J)= ONE
                 DX(J)  = XU(J)-XL(J)
                 IF( J .LE. NGDIM ) THEN
                    MA(J)  = NG*MA(J-1)
                 ENDIF
  130         CONTINUE
          ENDIF
C
C---------------------------------------------------------------
C           Set size of subregions uniform
C---------------------------------------------------------------
          NDM   = ND-1
          RC    = ONE/ND
          DO 155 J =1,NDIM
             K     = 0
             XN    = 0.
             DR    = XN
             I     = K
  140        K     = K+1
             DR    = DR+ONE
             XO    = XN
             XN    = XI(K,J)
  145       IF(RC .GT. DR) GO TO 140
             I     = I+1
             DR    = DR-RC
             XIN(I)= XN-(XN-XO)*DR
            IF(I .LT. NDM) GO TO 145
             DO 150 I  = 1,NDM
                XI(I,J)= XIN(I)
  150        CONTINUE
             XI(ND,J)  = ONE
  155     CONTINUE
C
      RETURN
      END
C
C***********************************************************************
C*========================                                             *
C* SUBROUTINE BSMAIN(FXN)                                              *
C*========================                                             *
C*        Main Program for the Integration progam BASES, which         *
C*     is new version of VEGAS.                                        *
C*     In terms of this program Integration can be done, furthermore   *
C*     a probability distribution can be made for the event generation.*
C*     The event geration is done by program SPRING.                   *
C*     For running BASES, there are four options.                      *
C*         IFLAG =  0 : First trial for defining grids.                *
C*         IFLAG =  1 : First trial for event accumulation.            *
C*         IFLAG =  2 : Second or more trial for defining grids.       *
C*         IFLAG =  3 : Second or more trial for accumulation.         *
C*                                                                     *
C*((Input)) from Unit 5 as card image                                  *
C*     NLOOP, MLOOP          Current loop count, Maximum loop count    *
C*     NPRINT                print option flag                         *
C*     IFLAG                 BASES input flag                          *
C*     SETIME                CPU time limit in minute                  *
C*      At the end of job, the job termination code is given, if it    *
C*      terminates by time limit. For the next submission the next     *
C*      loop count and input flag shuld be given.                      *
C*     NDIM  : Dimension of the integration                            *
C*     NTRIAL: Number of sampling points per cube. normally 2.         *
C*     ITMX* : Number of iteration                                     *
C*     ACC*  : Required accuracies                                     *
C*     XL(10): Lower limits of the integration variabels               *
C*     XU(10): upper limits of the integration variabels               *
C*                                                                     *
C*       Coded by S.Kawabata         May '85                           *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BSMAIN( FXN )
C
      IMPLICIT REAL*8 (A-H,O-Z)
      EXTERNAL FXN
      REAL*4 SETIM,TIME,RTIME,UTIME
      PARAMETER (MXDIM = 25)
      COMMON /BASE0/ SETIM,UTIME,IFLAG,NPRINT
      COMMON /BASE1/ XL(MXDIM),XU(MXDIM),NDIM,NGDIM,
     .               IG(MXDIM),NCALL
      COMMON /BASE2/ ACC1,ACC2,ITMX1,ITMX2
      COMMON /LOOP0/ LOOPC,IRCODE
      COMMON /LOOP1/ LOOP,MXLOOP
      COMMON /XHCNTL/ LOCK
 
C
C ---------------------------------------------------------------
C          Initialize timer
C ---------------------------------------------------------------
C
       CALL BSTIME( TIME, 0 )
C
C ---------------------------------------------------------------
C          Set BASES parameters equal to default values
C ---------------------------------------------------------------
C
       LOCK   =  0
       NDIM   = -1
       NGDIM  =  1
       ITMX1  = 15
       ITMX2  = 100
       NCALL  = 1000
       ACC1   = 0.2
       ACC2   = 0.01
       DO 100 J = 1,MXDIM
          IG(J) = 1
  100  CONTINUE
 
       READ( 5, * ) NLOOP, MLOOP
C
       READ( 5, * ) NPRINT
C
       READ( 5, * ) IFLAG
C
       READ( 5, * ) SETIM
C
       IF(   IFLAG .EQ.   0 ) THEN
 
               IF( NLOOP .LE. 0 ) THEN
                   WRITE(6,9000) NLOOP
 9000              FORMAT(1X,'*** ILLEGAL LOOP COUNT (',I3,' ) ***')
                   STOP
               ENDIF
 
               MNLOOP = NLOOP
               MXLOOP = MLOOP
               LOOPC  = NLOOP
 
               CALL BSUSRI
 
       ELSEIF( IFLAG .GT. 0 ) THEN
 
C         Read Temporary Result from Disk File
 
C                   -----------
                    CALL BSREAD
C                   -----------
                    MNLOOP = LOOP
                    LOOPC  = LOOP
                    LOCK   = 1
                    CALL BSUSRI
                    LOCK   = 0
 
       ELSE
           WRITE(6,9100) IFLAG
 9100      FORMAT(1X,'***** ILLEGAL INPUT IFAG (',I3,' ) *****')
           STOP
       ENDIF
C
       WRITE(6,9200) MNLOOP,MXLOOP,NPRINT,IFLAG,SETIM
 9200  FORMAT(/1H1,//1H ,20X,
     .       '*************************************',
     .  /21X,'*   Input Parameters for this JOB   *',
     .  /21X,'*                                   *',
     .  /21X,'*  Current Loop Count =',I12     ,' *',
     .  /21X,'*  Maximum Loop Count =',I12     ,' *',
     .  /21X,'*  Print Flag         =',I12     ,' *',
     .  /21X,'*  JOB Input Flag     =',I12     ,' *',
     .  /21X,'*  CPU Time Limit     =',F12.3   ,' *',
     .  /21X,'*                                   *',
     .  /21X,'*************************************')
C
       SETIM  = SETIM*60.0 - 5.
C
C
       DO 5000 LOOP = MNLOOP, MXLOOP
 
C
C  -----------------------------------------------------
C     Defining grids or just for integration
C  -----------------------------------------------------
             IF(IFLAG.LE.0.OR.IFLAG.EQ.2) THEN
C
                 CALL BASES( FXN )
                 IF(IFLAG.NE.1) GO TO 7000
C
                 CALL BSTIME( TIME, 1 )
                 RTIME = SETIM - TIME
                 IF(RTIME .LT. UTIME ) GO TO 7000
             ENDIF
C
C  ----------------------------------------------------
C     Accumulation to make probability distribution
C  ----------------------------------------------------
C
             CALL BASES( FXN )
C
             IF( IFLAG .NE. 0 ) GO TO 7000
C
             IF( NPRINT .LE. 0 )  CALL USROUT
C
             CALL BSTIME( TIME, 1 )
             RTIME = SETIM - TIME
 
             IF(LOOP .EQ. MXLOOP ) GO TO 8000
             IF(RTIME .LT. UTIME ) GO TO 6000
 
             LOOPC = LOOP + 1
             CALL BSUSRI
 
 5000  CONTINUE
C
       GO TO 8000
C
 6000  LOOP  = LOOP + 1
C
 7000  WRITE(6,9500)  LOOP, IFLAG
 9500  FORMAT(/1H1,20X,
     .    '****** CPU Time out  *********',
     ./21X,'  Next Loop Count =',I5,
     ./21X,'  Next Input Flag =',I5)
       GO TO 8500
C
 8000  WRITE(6,9600)  MXLOOP
 9600  FORMAT(/1H1,20X,
     .    '****** END OF BASES LOOP *********',
     ./21X,'  MAX. LOOP COUNT =',I5)
C
 8500  CALL BSWRIT
C
      RETURN
      END
C***********************************************************************
C*                                                                     *
C*========================                                             *
C*    SUBROUTINE BSREAD                                                *
C*========================                                             *
C*((Function))                                                         *
C*    READ TEMPORARY RESULT FROM DISK FILE                             *
C*((Auther))                                                           *
C*    S.Kawabata    June '90 at KEK                                    *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BSREAD
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXDIM = 25, NDMX = 50, LENG = 17000)
      PARAMETER (ITM  = 100 )
      COMMON /BASE1/ ND1(5*MXDIM+3)
      COMMON /BASE2/ ND2(6)
      COMMON /BASE3/ ND3(14)
      COMMON /BASE4/ ND4(2*MXDIM*(NDMX+1)+4*LENG+MXDIM+3)
      COMMON /BASE5/ ND5(9*ITM)
      COMMON /RANDM/ ND6(45)
      PARAMETER ( ILH = 50, IDH = 50 )
      COMMON /PLOTH/ NPH(9*(ILH+IDH)+2),NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ IBUF( ISL )
      COMMON /LOOP1/ LP1(2)
 
      REWIND 23
      READ(23) ND1,ND2,ND3,ND4,ND5,ND6,NPH,LP1
      READ(23) NW,(IBUF(I),I=1,NW)
 
      RETURN
      END
C
C
C***********************************************************************
C*===================                                                  *
C* SUBROUTINE BSUSRI                                                   *
C*===================                                                  *
C*((Purpose))                                                          *
C*        Call USERIN and check user's initialization parameters.      *
C*                                                                     *
C*       Coded by S.Kawabata        Oct. '85                           *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BSUSRI
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER ( MXDIM = 25)
      COMMON /BASE1/ XL(MXDIM),XU(MXDIM),NDIM,NGDIM,
     .               IG(MXDIM),NCALL
      COMMON /LOOP0/ LOOPC,IRCODE
 
C
             DO 100 I = 1,MXDIM
               XU(I)  = -10000.0D0
  100        CONTINUE
C
             CALL USERIN
C
             IF( NDIM .LT. 1) THEN
                WRITE(6,9200)
 9200           FORMAT(1X,'Error in USERIN; NDIM was not set.')
                STOP
             ENDIF
             DO 200 I = 1,NDIM
                 IF( XU(I) .LE. -10000.0) THEN
                    WRITE(6,9400) I,I
 9400               FORMAT(1X,'Error in USERIN; XL(',I2,
     .                     '), XU(',I2,') were not set.')
                    STOP
                 ENDIF
  200        CONTINUE
C
             IF( NGDIM .LT.  0) THEN
                 NGDIM = MIN( NDIM, 10)
             ELSEIF( NGDIM .GT. 10) THEN
                WRITE(6,9500) NGDIM
 9500           FORMAT(1X,'Error in USERIN; Too large NGDIM',
     .                /1X,'NGDIM must be less than 11 ||')
                STOP
             ENDIF
C
      RETURN
      END
C***********************************************************************
C*                                                                     *
C*========================                                             *
C*    SUBROUTINE BSWRIT                                                *
C*========================                                             *
C*((Purpose))                                                          *
C*    Read temporary result from disk file.                            *
C*((Auther))                                                           *
C*    S.Kawabata  June '90 at KEK                                      *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BSWRIT
 
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXDIM = 25, NDMX = 50, LENG = 17000)
      PARAMETER (ITM  = 100 )
      COMMON /BASE1/ ND1(5*MXDIM+3)
      COMMON /BASE2/ ND2(6)
      COMMON /BASE3/ ND3(14)
      COMMON /BASE4/ ND4(2*MXDIM*(NDMX+1)+4*LENG+MXDIM+3)
      COMMON /BASE5/ ND5(9*ITM)
      COMMON /RANDM/ ND6(45)
      PARAMETER ( ILH = 50, IDH = 50 )
      COMMON /PLOTH/ NPH(9*(ILH+IDH)+2),NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ IBUF( ISL )
      COMMON /LOOP1/ LP1(2)
 
      REWIND 23
      WRITE(23) ND1,ND2,ND3,ND4,ND5,ND6,NPH,LP1
      IF(NW .EQ. 0 ) NW = ISL
      WRITE(23) NW,(IBUF(I),I=1,NW)
C
      RETURN
      END
C
C***********************************************************************
C*=================================                                    *
C*    SUBROUTINE DHFILL(ID,DX,DY,FX)                                   *
C*=================================                                    *
C*((Function))                                                         *
C*    To fill scatter plot                                             *
C*  This routine identifies the bin number which is to be updated      *
C*  with weight FX*WGT.  Up to five points per plot are able to        *
C*  be stacked before calling BHUPDT or SHUPDT.                        *
C*((Input))                                                            *
C*  ID    : Histogram identification number                            *
C*  DX    : Input x value                                              *
C*  DY    : Input y value                                              *
C*  FX    : Input value of the function                                *
C*((Author))                                                           *
C*  S.Kawabata         June '90 at KEK                                 *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE DHFILL(ID,DX,DY,FX)
C
      REAL*8 DX, DY, FX
      REAL*8 SI,SI2,SWGT,SCHI,SCALLS,ATACC,WGT
      COMMON /BASE0/ SETIM,UTIME,IFLAG,NPRINT,IBASES
      COMMON /BASE3/ SI,SI2,SWGT,SCHI,SCALLS,ATACC,NSU,IT,WGT
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
 
      IF( NSCAT .GT. 0 ) THEN
          I  = IABS(MOD( ID, 13 )) + 1
          IF( DHASH(1, I) .EQ. 1 ) THEN
            IF( ID .EQ. MAPD( 1, DHASH(2,I))) THEN
                ISCAT = DHASH(2,I)
                GO TO 200
            ENDIF
          ELSEIF( DHASH(1, I) .GT. 1 ) THEN
            DO 100 K = 2, DHASH(1,I)+1
               IF( ID .EQ. MAPD( 1, DHASH(K,I))) THEN
                   ISCAT = DHASH(K,I)
                   GO TO 200
               ENDIF
  100       CONTINUE
          ENDIF
      ENDIF
C     WRITE(6,9000) ID
C9000 FORMAT(1X,'No Scat_Plot corresponds to ID =',I5,
C    .      /1X,' This call is neglected !!!')
      RETURN
 
  200 X     = DX*1.0
      Y     = DY*1.0
 
          IP1   = MAPD(2,ISCAT)
          XMIN  = BUFF(IP1)
          XMAX  = BUFF(IP1+1)
          MXBIN = IBUF(IP1+2)
          DEV   = BUFF(IP1+3)
          IX    =   0
          IY    =   0
          IF( X .GE. XMIN .AND. X .LE. XMAX ) THEN
              IX   = INT( (X - XMIN)/DEV+ 1.0 )
              IF( IX .GT. MXBIN ) IX =   0
          ENDIF
C
          IF( IX .GT. 0 ) THEN
              YMIN  = BUFF(IP1+4)
              YMAX  = BUFF(IP1+5)
              MYBIN = IBUF(IP1+6)
              DEV   = BUFF(IP1+7)
              IF( Y .GE. YMIN .AND. Y .LE. YMAX ) THEN
                  IY   = INT((Y - YMIN)/DEV + 1.0)
                 IF( IY .GT. MYBIN ) THEN
                     IX  =  0
                     IY  =  0
                 ENDIF
              ENDIF
          ENDIF
 
      IF( IBASES .EQ. 1 ) THEN
          IF( IY .GT. 0 ) THEN
 
              IP2       = MAPD(3,ISCAT)
              IBUF(IP2) = SCALLS
              IP2       = IX + MXBIN*(IY - 1) + IP2
              BUFF(IP2) = BUFF(IP2) + FX*WGT
 
          ENDIF
 
      ELSE
 
          IP3         = MAPD(4,ISCAT)
          IBUF(IP3)   = IX
          IBUF(IP3+1) = IY
 
      ENDIF
 
      RETURN
      END
C***********************************************************************
C*===============================================================      *
C*SUBROUTINE DHINIT(ID,DXMIN,DXMAX,NXBIN,DYMIN,DYMAX,NYBIN,TNAME)      *
C*===============================================================      *
C*((Function))                                                         *
C*    To define a scatter plot                                         *
C*((Input))                                                            *
C*   ID   : scatter plot identification number                         *
C*   DXMIN: Lower limit of X for the scatter plot                      *
C*   DXMAX: Upper limit of X for the scatter plot                      *
C*   NXBIN: Number of bins of X for the plot (Max. is 50 )             *
C*   DYMIN: Lower limit of Y for the scatter plot                      *
C*   DYMAX: Upper limit of Y for the scatter plot                      *
C*   NYBIN: Number of bins of Y for the plot (Max. is 50 )             *
C*   TNAME: Title of the plot in the character string (upto 64         *
C*           characters)                                               *
C*((Author))                                                           *
C*   S.Kawabata     June '90 at KEK                                    *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE DHINIT(ID,DXMIN,DXMAX,NXBIN,DYMIN,DYMAX,NYBIN,TNAME)
C
      REAL*8 DXMIN,DXMAX,DYMIN,DYMAX
      CHARACTER*(*) TNAME
      CHARACTER*64 NAME
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
      COMMON/XHCNTL/ LOCK
 
      IF( LOCK .NE. 0 ) RETURN
 
      IF( NSCAT .GE. MAXD ) THEN
          WRITE(6,9000) NSCAT,ID
 9000     FORMAT(1X,'Numberof Scat_plots exceeds ',I3,' at ID = ',I3,
     .            /1X,'This call is neglected.')
          RETURN
      ENDIF
      I  = IABS(MOD( ID, 13 )) + 1
      IF( DHASH(1, I) .EQ. 1 ) THEN
            IF( ID .EQ. MAPD( 1, DHASH(2,I))) THEN
               WRITE(6,9100) ID
 9100          FORMAT(1X,'Scat Plot ID (',I3,' ) exists already.'
     .               /1X,' This call is neglected.')
               RETURN
            ENDIF
      ELSEIF( DHASH(1, I) .GT. 1 ) THEN
          DO 100 K = 2, DHASH(1,I)+1
            IF( ID .EQ. MAPD( 1, DHASH(K,I))) THEN
               WRITE(6,9100) ID
               RETURN
            ENDIF
  100    CONTINUE
      ENDIF
      XMIN  = DXMIN*1.0
      XMAX  = DXMAX*1.0
      YMIN  = DYMIN*1.0
      YMAX  = DYMAX*1.0
      IF(MAXD .GT. IDH ) THEN
         WRITE(6,9200) ID
 9200    FORMAT(1X,'Number of Scat_Plots exceeds 10 at ID =',I5,
     .         /1X,' This call is neglected.')
         RETURN
      ELSEIF(NXBIN .GT. 50 .OR. NYBIN .GT. 50 ) THEN
         WRITE(6,9300) NXBIN,NYBIN,ID
 9300    FORMAT(1X,'Bin size (',2I3,' )  exceeds 50 at ID =',I5,
     .         /1X,' This call is neglected .')
         RETURN
      ELSEIF(XMIN  .GE. XMAX .OR. YMIN .GE. YMAX) THEN
         WRITE(6,9400) ID,XMIN,XMAX,YMIN,YMAX
 9400    FORMAT(1X,'Lower limit is larger than upper at SC_PL ID =',I5,
     .         /1X,' This call is neglected .',
     .         /1X,' XMIN =',G13.4,' XMAX =',G13.4,
     .         /1X,' YMIN =',G13.4,' YMAX =',G13.4)
         RETURN
      ENDIF
      IF(DHASH(1,I) .GE. 49 ) THEN
         WRITE(6,9500) I
 9500    FORMAT(1X,I5,'-th Hash table overflow',
     .         /1X,' This call is neglected.')
         RETURN
      ENDIF
 
      NSCAT        = NSCAT + 1
 
      DHASH(1,I)   = DHASH(1,I) + 1
      K            = DHASH(1,I) + 1
      DHASH(K,I)   = NSCAT
 
      IP1    = NW + 1
         NW  = NW + 2527
         MAPD(1,NSCAT)  = ID
         MAPD(2,NSCAT)  = IP1
         BUFF(IP1     ) = XMIN
         BUFF(IP1 +  1) = XMAX
         IBUF(IP1 +  2) = NXBIN
         DEV            = XMAX - XMIN
         BUFF(IP1 +  3) = DEV/NXBIN
         BUFF(IP1 +  4) = YMIN
         BUFF(IP1 +  5) = YMAX
         IBUF(IP1 +  6) = NYBIN
         DEV            = YMAX - YMIN
         BUFF(IP1 +  7) = DEV/NYBIN
      IP2   = IP1 + 8
         MAPD(3,NSCAT)  = IP2
         IBUF(IP2     ) = 0
      IP3   = IP1 + 2509
         MAPD(4,NSCAT)  = IP3
         IBUF(IP3     ) =  0
         IBUF(IP3 +  1) =  0
 
         I1   = IP3 + 2
         I2   = I1 + 15
         NAME = TNAME
         READ(NAME,9800) (BUFF(I),I=I1,I2)
 9800    FORMAT(16A4)
 
      RETURN
      END
C
C***********************************************************************
C*=======================                                              *
C*    SUBROUTINE DHPLOT                                                *
C*=======================                                              *
C*((Purpose))                                                          *
C*     To print scatter plots for BASES and SPRING                     *
C*                                                                     *
C*((Author))                                                           *
C*      S.Kawabata    June '90                                         *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE DHPLOT
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
 
      CHARACTER*1  PLUS,MINUS,BLNK,STAR,NUM(0:9),NEG(0:9),SHARP,PNT
      REAL*4       X(50)
      CHARACTER*1 CHAR(50)
      CHARACTER*80 FORM1,FORM
      DATA  PLUS /'+'/, MINUS /'-'/, BLNK /' '/, STAR /'*'/
      DATA  SHARP /'#'/,  PNT /'.'/
      DATA  NUM  / '0','1','2','3','4','5','6','7','8','9'/
      DATA  NEG  / '-','a','b','c','d','e','f','g','h','i'/
C
C
      IF( NSCAT .LE. 0 ) THEN
         WRITE(6,9000)
 9000    FORMAT(1X,'No Scat_Plot')
      ELSE
         DO 900 ISCAT = 1, NSCAT
            IP3   = MAPD(4,ISCAT)
            WRITE(6,9100) MAPD(1,ISCAT),(BUFF(I), I=IP3+2,IP3+17)
 9100       FORMAT(/1H1,/1H ,
     .      5X,'Scat_Plot (ID =',I3,' ) for ',16A4,/)
            IP1   = MAPD(2,ISCAT)
            XL    = BUFF(IP1)
            XU    = BUFF(IP1+1)
            NX    = IBUF(IP1+2)
            DX    = BUFF(IP1+3)
            XM    = ABS(XU)
            XX    = ABS(XL)
            IF( XX .GT. XM ) XM = XX
            CALL XHORDR( XU, FX, XORD, IXORD)
            YL    = BUFF(IP1+4)
            YU    = BUFF(IP1+5)
            NY    = IBUF(IP1+6)
            DY    = BUFF(IP1+7)
            MIDY  = NY/2
            IF( MIDY .EQ. 0 ) MIDY = 1
            YM    = ABS(YU)
            YY    = ABS(YL)
            IF( YY .GT. YM ) YM = YY
            CALL XHORDR( YM, FY, YORD, IYORD)
            IP2   = MAPD(3,ISCAT)
            NTOTAL= IBUF(IP2)
            VMAX  = BUFF(IP2+1)
            VMIN  = VMAX
            DO 100 J = 0,NY-1
               IB    = NX*J + IP2
               DO 100 I = 1,NX
                  VLS    = BUFF( I + IB )
                  IF( VLS .GT. VMAX ) VMAX = VLS
                  IF( VLS .LT. VMIN ) VMIN = VLS
  100       CONTINUE
 
            IF( VMAX .GT. -VMIN ) THEN
                UNIT = ABS(VMAX)/11.0
            ELSE
                UNIT = ABS(VMIN)/11.0
            ENDIF
            WRITE(FORM1,9200) NX
 9200       FORMAT('(7X,''E'',I3,3X,''+'',',I2,'(''--''),''-+'')')
            WRITE(6,FORM1) IYORD
            DO 300 L = NY-1,0,-1
               IB     = NX*L + IP2
               DO 200 I = 1,NX
                 XNUM   = BUFF( I + IB )/UNIT
                 IF( XNUM .LT. 0 0 ) THEN
                     NUMB   = XNUM - 1.0
                     IF(     NUMB .GE. -1 )THEN
                             CHAR(I) = MINUS
                     ELSEIF( NUMB .GE. -10 ) THEN
                            CHAR(I)  = NEG(-NUMB-1)
                     ELSE
                            CHAR(I)  = SHARP
                     ENDIF
                 ELSE
                     NUMB   = XNUM + 1.0
                     IF(     XNUM .EQ. 0.0 ) THEN
                             CHAR(I) = BLNK
                     ELSEIF( NUMB .LE.  1 ) THEN
                             CHAR(I) = PLUS
                             IF( VMIN .GE. 0.0 ) CHAR(I) = PNT
                     ELSEIF( NUMB .LE. 10 ) THEN
                             CHAR(I) = NUM(NUMB-1)
                     ELSE
                             CHAR(I) = STAR
                     ENDIF
                 ENDIF
  200          CONTINUE
 
               Y   = (L*DY + YL)/YORD
               IF( L .EQ. MIDY ) THEN
                   WRITE(FORM,9300) NX
 9300              FORMAT('(5X,F6.3,'' Y I'',',I2,'(1X,A1),'' I'')')
               ELSE
                   WRITE(FORM,9310) NX
 9310              FORMAT('(5X,F6.3,''   I'',',I2,'(1X,A1),'' I'')')
               ENDIF
               WRITE(6,FORM) Y,(CHAR(M),M=1,NX)
 
  300       CONTINUE
 
            WRITE(6,FORM1) IYORD
 
            WRITE(FORM,9400) NX
 9400       FORMAT('(6X,''Low-'',5X,',I2,'X,''X'')')
            WRITE(6,FORM)
 
            XORD     = XORD*10.
            DO 400 I = 1, NX
               X(I)  = ((I-1)*DX + XL)/XORD
               IF( X(I) .LT. 0.0 ) THEN
                   CHAR(I)   = MINUS
                   X(I)      = -X(I)
               ELSE
                   CHAR(I)   = BLNK
               ENDIF
  400       CONTINUE
            WRITE(FORM1,9500) NX
 9500       FORMAT('(6X,''Edge'',5X,',I2,'(1X,A1))')
            WRITE(6,FORM1) (CHAR(M),M=1,NX)
 
            XORD      = 1.0
            DO 600 I  = 1,5
               IF( I .EQ. 2 ) THEN
                   WRITE(FORM,9602) NX
 9602              FORMAT('(7X,''E'',I3,4X',I2,
     .                    '('' .''))')
                   WRITE(6,FORM) IXORD
               ELSE
                   DO 500 J = 1, NX
                      XX        = X(J)*10.0
                      NUMB      = XX
                      CHAR(J)   = NUM(NUMB)
                      X(J)      = XX - FLOAT(NUMB)
  500              CONTINUE
                   IF(     I .EQ. 4 ) THEN
                           WRITE(FORM,9604) NX
 9604                      FORMAT('(7X,''Low-'',4X,',I2,
     .                            '(1X,A1))')
                   ELSEIF( I .EQ. 5 ) THEN
                           WRITE(FORM,9605) NX
 9605                      FORMAT('(7X,''Edge'',4X,',I2,
     .                            '(1X,A1))')
                   ELSE
                           WRITE(FORM,9601) NX
 9601                      FORMAT('(15X,',I2,
     .                            '(1X,A1))')
                   ENDIF
                   WRITE(6,FORM) (CHAR(M),M=1,NX)
               ENDIF
  600       CONTINUE
 
  900    CONTINUE
      ENDIF
C
      RETURN
      END
C**********************************************************************
C*============================                                        *
C* Subroutine DRNSET( ISEED )                                         *
C*============================                                        *
C*((Purpose))                                                         *
C*  Initialization routine of                                         *
C*         Machine-independent Random number generator                *
C*         General purpose Version,  OK as long as >= 32 bits         *
C*((Arguement))                                                       *
C*  ISEED: SEED                                                       *
C*                                                                    *
C**********************************************************************
 
      SUBROUTINE DRNSET( ISEED )
 
      COMMON/RANDM/RDM(31),RM1,RM2,IA1,IC1,M1,IX1,
     .                             IA2,IC2,M2,IX2,
     .                             IA3,IC3,M3,IX3
 
      IA1 =    1279
      IC1 =  351762
      M1  = 1664557
      IA2 =    2011
      IC2 =  221592
      M2  = 1048583
      IA3 =   15091
      IC3 =    6171
      M3  =   29201
 
C Initialization
 
      IX1  = MOD( ISEED, M1 )
      IX1  = MOD( IA1*IX1+IC1, M1 )
      IX2  = MOD( IX1, M2 )
      IX1  = MOD( IA1*IX1+IC1, M1 )
      IX3  = MOD( IX1,M3)
      RM1  = 1./FLOAT(M1)
      RM2  = 1./FLOAT(M2)
      DO 100 J = 1,31
         IX1   = MOD( IA1*IX1+IC1, M1 )
         IX2   = MOD( IA2*IX2+IC2, M2 )
         RDM(J)= ( FLOAT(IX1)+FLOAT(IX2)*RM2 )*RM1
  100 CONTINUE
 
      RETURN
      END
C**********************************************************************
C*======================                                              *
C* FUNCTION DRN( ISEED)                                               *
C*======================                                              *
C*  Machine-independent Random number generator                       *
C*     General purpose Version,  OK as long as >= 32 bits             *
C*((Arguement))                                                       *
C*  ISEED: Seed                                                       *
C*                                                                    *
C**********************************************************************
 
      DOUBLE PRECISION FUNCTION DRN(ISEED)
 
      COMMON/RANDM/RDM(31),RM1,RM2,IA1,IC1,M1,IX1,
     .                             IA2,IC2,M2,IX2,
     .                             IA3,IC3,M3,IX3
 
C Generate Next number in sequence
 
      IX1    = MOD( IA1*IX1+IC1, M1 )
      IX2    = MOD( IA2*IX2+IC2, M2 )
      IX3    = MOD( IA3*IX3+IC3, M3 )
      J      = 1 + (31*IX3)/M3
      DRN    = RDM(J)
      RDM(J) = ( FLOAT(IX1)+FLOAT(IX2)*RM2 )*RM1
 
C Omit following statement if function arguement passed by value:
 
      ISEED = IX1
      RETURN
      END
C
C***********************************************************************
C*=========================                                            *
C*    SUBROUTINE XHCHCK                                                *
C*=========================                                            *
C*((Purpose))                                                          *
C*     To check the contents of the histogram table                    *
C*                                                                     *
C*((Author))                                                           *
C*      S.Kawabata    June '90                                         *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE XHCHCK
C
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
 
      WRITE(6,9000) NW
 9000 FORMAT(/1H1,//1H ,
     .   5X,'*********  Contents of the histogram Header *********',
     .     //1X,'(1) Actual Buffer size     = ',I6,' Words')
      WRITE(6,9100) MAXL,NHIST
 9100 FORMAT(1X,'(2) Contents of Histograms ',
     .      /1X,'    Max. No. of Histograms = ',I6,
     .      /1X,'    Number   of Histograms = ',I6)
 
      IF( NHIST .GT. 0 ) THEN
          WRITE(6,9200)
 9200     FORMAT(1X,'   ID     X_min        X_max    X_bin',
     .              ' Hash Hst#')
          DO 200 I = 1, 13
             NT    = XHASH(1,I)
             IF( NT .GT. 0 ) THEN
                 DO 100 J = 2, NT+1
                    K     = XHASH(J,I)
                    IP1   = MAPL(2,K)
                    IP3   = MAPL(4,K)
                    XMIN  = BUFF(IP1)
                    XMAX  = BUFF(IP1+1)
                    NBIN  = IBUF(IP1+2)
                    WRITE(6,9300) MAPL(1,K),XMIN,XMAX,NBIN,I,NT,K
 9300               FORMAT(1X,I5,1X,1PE12.4,1X,E12.4,I5,2I3,I5)
  100            CONTINUE
             ENDIF
  200     CONTINUE
      ENDIF
 
      WRITE(6,9400) MAXD,NSCAT
 9400 FORMAT(1X,'(3) Contents of Scatter Plots',
     .      /1X,'    Max. No. of Scat_Plots = ',I6,
     .      /1X,'    Number   of Scat_Plots = ',I6)
 
      IF( NSCAT .GT. 0 ) THEN
          WRITE(6,9500)
 9500     FORMAT(1X,'   ID      X_min   ',
     .              '     X_max   X-Bin    Y_min   ',
     .              '     Y_max   Y_Bin Hash Hst#')
          DO 400 I = 1, 13
             NT    = DHASH(1,I)
             IF( NT .GT. 0 ) THEN
                 DO 300 J = 2, NT+1
                    K     = DHASH(J,I)
                    IP1   = MAPD(2,K)
                    IP3   = MAPD(4,K)
                    XMIN  = BUFF(IP1)
                    XMAX  = BUFF(IP1+1)
                    NXBN  = IBUF(IP1+2)
                    YMIN  = BUFF(IP1+4)
                    YMAX  = BUFF(IP1+5)
                    NYBN  = IBUF(IP1+6)
                    WRITE(6,9600) MAPD(1,K),XMIN,XMAX,NXBN,
     .                            YMIN,YMAX,NYBN,I,NT,K
 9600               FORMAT(1X,I5,1X,1PE12.4,1X,E12.4,I5,
     .                                 E12.4,1X,E12.4,I5,2I3,I5)
  300            CONTINUE
             ENDIF
  400     CONTINUE
      ENDIF
      RETURN
      END
C***********************************************************************
C*=================================                                    *
C*    SUBROUTINE XHFILL(ID, DX, FX)                                    *
C*=================================                                    *
C*((Function))                                                         *
C*    To fill histograms.                                              *
C*  This routine identifies the bin number which is to be updated      *
C*  with weight FX*WGT.  Up to five points per histogram are able      *
C*  to be stacked before calling BHUPDT or SHUPDT.                     *
C*((Input))                                                            *
C*  ID    : Histogram identification number                            *
C*  DX    : Input value                                                *
C*  FX    : Input value of the function                                *
C*((Author))                                                           *
C*  S.Kawabata         June '90 at KEK                                 *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE XHFILL(ID, DX, FX )
C
      REAL*8 DX,FX
      REAL*8 SI,SI2,SWGT,SCHI,SCALLS,ATACC,WGT
      COMMON /BASE0/ SETIM,UTIME,IFLAG,NPRINT,IBASES
      COMMON /BASE3/ SI,SI2,SWGT,SCHI,SCALLS,ATACC,NSU,IT,WGT
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
C
      IF( NHIST .GT. 0 ) THEN
C
          I  = IABS(MOD( ID, 13 )) + 1
          IF( XHASH(1, I) .EQ. 1 ) THEN
            IF( ID .EQ. MAPL( 1, XHASH(2,I))) THEN
                IHIST = XHASH(2,I)
                GO TO 200
            ENDIF
          ELSEIF( XHASH(1, I) .GT. 1 ) THEN
            DO 100 K = 2, XHASH(1,I)+1
               IF( ID .EQ. MAPL( 1, XHASH(K,I))) THEN
                   IHIST = XHASH(K,I)
                   GO TO 200
               ENDIF
  100       CONTINUE
          ENDIF
      ENDIF
 
C     WRITE(6,9000) ID
C9000 FORMAT(1X,'No Histogram corresponds to ID =',I5,
C    .      /1X,' This call is neglected.')
      RETURN
C
 
  200 X     = DX*1.0
 
          IX    = -1
          IP1   = MAPL(2,IHIST)
          XMIN  = BUFF(IP1)
          XMAX  = BUFF(IP1+1)
          NXBIN = IBUF(IP1+2)
          DEV   = BUFF(IP1+3)
          IF(     X .LT. XMIN ) THEN
                  IX   = 0
          ELSEIF( X .GT. XMAX ) THEN
                 IX   = NXBIN + 1
          ELSE
                 IX   = INT((X - XMIN)/DEV + 1.0)
                 IF( IX .GT. NXBIN ) IX = NXBIN
          ENDIF
 
      IF( IBASES .EQ. 1 ) THEN
 
          IP2       = MAPL(3,IHIST) + IX
          IBUF(IP2) = IBUF(IP2) + 1
          FXWGT     = FX*WGT
          IP2       = IP2 + 52
          BUFF(IP2) = BUFF(IP2) + FXWGT
          IP2       = IP2 + 52
          BUFF(IP2) = BUFF(IP2) + FXWGT*FXWGT
 
      ELSE
 
         IP3        =  MAPL(4,IHIST)
         IBUF(IP3)  = IX
 
      ENDIF
 
C
      RETURN
      END
C   =====================================
      SUBROUTINE XHGRID( LUNIT )
C   =====================================
C
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (MXDIM = 25, NDMX = 50, LENG = 17000)
      COMMON /BASE1/ XL(MXDIM),XU(MXDIM),NDIM,NGDIM,
     .               IG(MXDIM),NCALL
      COMMON /BASE4/ XI(NDMX,MXDIM),DX(MXDIM),DXD(LENG),DXP(LENG),
     .               ND,NG,NPG,MA(MXDIM)
      COMMON /BASE6/ D(NDMX,MXDIM)
 
      CHARACTER*50 WINDOW(8)
      DATA WINDOW /'SET WINDOW X FROM  1.0 TO  5.0 Y FROM  8.0 TO 11.5',
     .             'SET WINDOW X FROM  5.5 TO  9.5 Y FROM  8.0 TO 11.5',
     .             'SET WINDOW X FROM 10.0 TO 14.0 Y FROM  8.0 TO 11.5',
     .             'SET WINDOW X FROM 14.5 TO 18.5 Y FROM  8.0 TO 11.5',
     .             'SET WINDOW X FROM  1.0 TO  5.0 Y FROM  3.0 TO  6.5',
     .             'SET WINDOW X FROM  5.5 TO  9.5 Y FROM  3.0 TO  6.5',
     .             'SET WINDOW X FROM 10.0 TO 14.0 Y FROM  3.0 TO  6.5',
     .             'SET WINDOW X FROM 14.5 TO 18.5 Y FROM  3.0 TO  6.5'/
      REAL*4 X(8),Y(8)
      DATA X     /  2.0, 6.5, 11.0, 15.5, 2.0, 6.5, 11.0, 15.5 /
      DATA Y     /  4*7.5, 4*2.5/
 
      NO     = 0
      XND    = FLOAT(ND)
      MPAGE  = (NDIM-1)/8 + 1
      DO 1000 NPAGE = 1, MPAGE
 
         NFIGS = 8
         IF( NPAGE .EQ. MPAGE ) NFIGS = MOD(NDIM, 8)
         IF( NFIGS .GT. 0 ) THEN
 
            WRITE( LUNIT, 9000)
 9000       FORMAT(1X,'NEW FRAME')
            WRITE( LUNIT, 9001)
 9001       FORMAT(1X,'SET FONT DUPLEX')
            WRITE( LUNIT, 9002)
 9002       FORMAT(1X,'SET SIZE 20 BY 14.00')
 
            DO 500 NF = 1, NFIGS
               NO    = NO + 1
               WRITE( LUNIT, 9003) WINDOW(NF)
 9003          FORMAT(1X,A)
               WRITE( LUNIT, 9004) XND
 9004          FORMAT(1X,'SET LIMIT X FROM 0.0 TO ',G12.4,
     .               /1X,'SET LABEL LEFT OFF')
               WRITE( LUNIT, 9005)
 9005          FORMAT(1X,'SET INTENSITY 4',
     .               /1X,'SET TICK SIZE 0.04',
     .               /1X,'SET ORDER X Y ')
               DO 200 I = 1, ND
                  WRITE( LUNIT, 9006) FLOAT(I),D(I,NO)
 9006             FORMAT(1X,2G12.4)
  200          CONTINUE
               WRITE( LUNIT, 9007)
C 9007          FORMAT(1X,'HIST')
 9007          FORMAT(1X,'PLOT')
               WRITE( LUNIT, 9008 ) X(NF),Y(NF),NF
 9008          FORMAT(1X,'TITLE ',2G12.4,''' Dim # ', I3)
  500       CONTINUE
         ENDIF
 1000 CONTINUE
 
      RETURN
      END
C***********************************************************************
C*===============================================                      *
C*    SUBROUTINE XHINIT(ID,DXMIN,DXMAX,NBIN,TNAME)                     *
C*===============================================                      *
C*((Function))                                                         *
C*    To define a histogram.                                           *
C*((Input))                                                            *
C*   ID   : Histogram identification number                            *
C*   DXMIN: Lower limit of the histogram                               *
C*   DXMAX: Upper limit of the histogram                               *
C*   NBIN : Number of bins for the histogram (Max. is 50 )             *
C*   TNAME: Title of the histogram in the character string (upto 64    *
C*           characters)                                               *
C*((Author))                                                           *
C*   S.Kawabata    June '90                                            *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE XHINIT(ID,DXMIN,DXMAX,NBIN,TNAME)
C
      REAL*8 DXMIN, DXMAX
      CHARACTER*(*) TNAME
      CHARACTER*68  NAME
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
      COMMON/XHCNTL/ LOCK
C
      IF( LOCK .NE. 0 ) RETURN
 
      IF((NHIST .GE. MAXL-1) .AND. (ID .GT. 0) ) THEN
         WRITE(6,9000) MAXL-1,ID
 9000    FORMAT(1X,'Number of Histograms exceeds',I3,' at ID =',I5,
     .         /1X,' This call is neglected.')
         RETURN
      ENDIF
 
      I  = IABS(MOD( ID, 13 )) + 1
      IF( XHASH(1, I) .EQ. 1 ) THEN
            IF( ID .EQ. MAPL( 1, XHASH(2,I))) THEN
               WRITE(6,9100) ID
 9100          FORMAT(1X,'HISTOGRAM ID (',I3,' ) exists already.'
     .               /1X,' This call is neglected.')
               RETURN
            ENDIF
      ELSEIF( XHASH(1, I) .GT. 1 ) THEN
          DO 100 K = 2, XHASH(1,I)+1
            IF( ID .EQ. MAPL( 1, XHASH(K,I))) THEN
               WRITE(6,9100) ID
               RETURN
            ENDIF
  100    CONTINUE
      ENDIF
 
      XMIN  = DXMIN*1.0
      XMAX  = DXMAX*1.0
      IF(NBIN  .GT. 50 ) THEN
         WRITE(6,9200) NBIN,ID
 9200    FORMAT(1X,'Bin size (',I3,' )  exceeds 50 at ID =',I5,
     .         /1X,' This call is neglected.')
         RETURN
      ENDIF
      IF(XMIN  .GE. XMAX ) THEN
         WRITE(6,9300) ID
 9300    FORMAT(1X,'Lower limit is larger than upper at ID =',I5,
     .         /1X,' This call is neglected.')
         RETURN
      ENDIF
      IF(XHASH(1,I) .GE. 49 ) THEN
         WRITE(6,9400) I
 9400    FORMAT(1X,I5,'-th Hash table overflow',
     .         /1X,' This call is neglected.')
         RETURN
      ENDIF
 
      NHIST        = NHIST + 1
      XHASH(1,I)   = XHASH(1,I) + 1
      K            = XHASH(1,I) + 1
      XHASH(K,I)   = NHIST
 
      IP1    = NW + 1
         NW  = NW + 281
         MAPL(1,NHIST)  = ID
         MAPL(2,NHIST)  = IP1
         BUFF(IP1     ) = XMIN
         BUFF(IP1 +  1) = XMAX
         IBUF(IP1 +  2) = NBIN
         DEV            = XMAX - XMIN
         BUFF(IP1 +  3) = DEV/NBIN
      IP2   = IP1 + 4
         MAPL(3,NHIST)  = IP2
      IP3   = IP1 + 264
         MAPL(4,NHIST)  = IP3
         IBUF(IP3)      = -1
 
         I1   = IP3 + 1
         I2   = I1 + 15
         NAME = TNAME
         READ(NAME,9800) (BUFF(I),I=I1,I2)
 9800    FORMAT(16A4)
 
C
 1000 CONTINUE
      RETURN
      END
C***********************************************************************
C*                                                                     *
C*=============================================                        *
C*    SUBROUTINE XHORDR( VAL, F2, ORDER, IORDR)                        *
C*=============================================                        *
C*((Function))                                                         *
C*    To resolve the real number VAL into mantester and exponent parts.*
C*  When VAL = 1230.0 is given, output are                             *
C*        F2 = 1.2  and ORDER = 4.0.                                   *
C*((Input))                                                            *
C*  VAL  : Real*4 value                                                *
C*((Output))                                                           *
C*  F2   : The upper two digits is given                               *
C*  ORDER: Order is given                                              *
C*  IORDR: Exponent is given                                           *
C*((Author))                                                           *
C*  S.Kawabata                                                         *
C*                                                                     *
C***********************************************************************
 
      SUBROUTINE XHORDR(VAL, F2, ORDER, IORDR)
 
      IF( VAL .NE. 0.0 ) THEN
          ORDER    =  LOG10( VAL )
          IORDR    =  INT( ORDER )
          IF( ORDER .LT. 0.0 ) IORDR = IORDR - 1
          ORDER  = 10.0**IORDR
          F2     = VAL/ORDER
      ELSE
          IORDR  = 0
          ORDER  = 1.0
          F2    = 0.0
      ENDIF
 
      RETURN
      END
C***********************************************************************
C*======================================                               *
C*    SUBROUTINE XHPLOT( IFG,  IHIST )                                 *
C*=====================================                                *
C*((Purpose))                                                          *
C*     To print histograms for BASES and SPRING.                       *
C*((Input))                                                            *
C*     IFG  : Flag which indicats whether this is called by BASES      *
C*            or SPRING.  IFG = ( 0 / anyother) = ( By BASES/ SPRING)  *
C*     IHIST: Serial number of the histogram                           *
C*                                                                     *
C*((Author))                                                           *
C*      S.Kawabata    June '90 at KEK                                  *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE XHPLOT( IFG, IHIST )
C
      CHARACTER*80 CHAR,CHAR1
      REAL*8         SI,SI2,SWGT,SCHI,SCALLS,ATACC,WGT
      COMMON /BASE3/ SI,SI2,SWGT,SCHI,SCALLS,ATACC,NSU,IT,WGT
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
 
      REAL  VAL(0:51),VLOG(0:51),VERR(0:51),SCALE(10)
      CHARACTER*1  BLNK,STAR,OO,AI
      CHARACTER*91 FORM
      DATA  YMAX / 80/
      DATA  BLNK /' '/, STAR /'*'/, OO /'O'/, AI /'I'/
 
      IP3   = MAPL(4,IHIST)
      IF(     IFG .EQ. 0 ) THEN
            WRITE(6,9100) MAPL(1,IHIST),(BUFF(I),I=IP3+1,IP3+16)
 9100       FORMAT(/1H1,
     .      5X,'Histogram (ID =',I3,' ) for ',16A4)
      ELSEIF( IFG .EQ. -10 ) THEN
            WRITE(6,9102) (BUFF(I),I=IP3+1,IP3+16)
 9102       FORMAT(/1H1,5X,16A4)
      ELSE
            WRITE(6,9105) MAPL(1,IHIST),(BUFF(I),I=IP3+1,IP3+16)
 9105       FORMAT(/1H1,
     .      5X,'Additional Histogram (ID =',I3,' ) for ',16A4)
      ENDIF
 
      IP1   = MAPL(2,IHIST)
      XMIN  = BUFF(IP1)
      XMAX  = BUFF(IP1+1)
      NXBIN = IBUF(IP1+2) + 1
      DEV   = BUFF(IP1+3)
      IP2   = MAPL(3,IHIST)
 
      IF( IFG .EQ. 0 ) THEN
          NTOTAL     = SCALLS
          FACT       = 1./(NTOTAL*DEV)
          IPF   = IP2 + 156
          IPF2  = IPF + 52
          VAL(0)     = BUFF(IPF)/NTOTAL
          VAL(NXBIN) = BUFF(IPF+NXBIN)/NTOTAL
          VMAX       = FACT*BUFF(IPF+1)
          VMIN       = VMAX
          DO  50 I   = 1,NXBIN-1
              TX     = BUFF(I+IPF)
              NX     = IBUF(I+IP2)
              VLS    = TX*FACT
              IF( VMAX .LT. VLS ) VMAX = VLS
              IF( VMIN .GT. VLS ) VMIN = VLS
              VAL(I) = VLS
              IF( NX .GT. 1 ) THEN
                  DEV2   =  NX*BUFF(I+IPF2)-TX*TX
                  IF( DEV2 .LE. 0.0 ) THEN
                      VERR(I)= 0.0
                  ELSE
                      VERR(I)= FACT*SQRT( DEV2/( NX-1 ))
                  ENDIF
              ELSEIF( NX .EQ. 1 ) THEN
                  VERR(I)= VLS
              ELSE
                  VERR(I)= 0.0
              ENDIF
 
   50     CONTINUE
      ELSE
          IPX   = IP2 + 52
          VAL(0)     = BUFF(IPX)
          VAL(NXBIN) = BUFF(IPX+NXBIN)
          NTOTAL     = VAL(0) + VAL(NXBIN)
          VMIN       = 0.0
          VMAX       = VMIN
          DO  55 I   = 1,NXBIN-1
              VLS    = BUFF(I+IPX)
              NTOTAL = VLS + NTOTAL
              IF( VMAX .LT. VLS ) VMAX = VLS
              VAL(I) = VLS
              IF( VLS .GT. 0.0 ) THEN
                  VERR(I) = SQRT(VLS)
              ELSE
                  VERR(I) = 0.0
              ENDIF
   55     CONTINUE
       ENDIF
 
       IF( VMIN .GE. 0.0 ) THEN
           VLMAX = ALOG10(VMAX)
           VLMIN = VLMAX
           DO  60 I = 0,NXBIN
               IF( VAL(I) .GT. 0.0 ) THEN
                   VLS   = LOG10( VAL(I) )
                   IF( I .GT. 0 .AND. I .LT. NXBIN ) THEN
                       IF( VLS .LT. VLMIN ) VLMIN = VLS
                   ENDIF
                   VLOG(I)  = VLS
               ENDIF
   60      CONTINUE
 
           IF( VLMIN .LT. 0.0) THEN
               VXMIN = IFIX(VLMIN) - 1.0
           ELSE
               VXMIN = IFIX(VLMIN)
           ENDIF
           VXMAX = VLMAX
           IFLG  = 1
           CALL XHRNGE( IFLG, VXMIN, VXMAX, VLMIN, VLMAX, VLSTP )
           UNITL = (VLMAX-VLMIN)/YMAX
 
       ENDIF
 
       IFLG   = 0
       IF( VMAX .GT. 0.0 ) THEN
           IF( VMIN .GE. 0.0 ) THEN
               VXMAX  = 1.2*VMAX
               VXMIN  = 0.0
               CALL XHRNGE( IFLG, VXMIN, VXMAX, VMIN, VMAX, VSTP )
           ELSE
               VXMAX  = 1.1*VMAX
               VXMIN  = 1.1*VMIN
               CALL XHRNGE( IFLG, VXMIN, VXMAX, VMIN, VMAX, VSTP )
           ENDIF
       ELSE
          VXMAX  = 0.0
          VXMIN  = 1.1*VMIN
          CALL XHRNGE( IFLG, VXMIN, VXMAX, VMIN, VMAX, VSTP )
       ENDIF
 
       UNIT  = (VMAX-VMIN)/YMAX
 
       CALL XHSCLE( IFLG, VMIN, VMAX, VSTP, UNIT,
     .                          NSCL, NBLK, SCALE, CHAR1)
C
C
       MXNSC = 74/(NBLK+10)
       IF( NSCL .GT. MXNSC) NSCL = MXNSC
 
       IF( IFG .EQ. 0 ) THEN
             WRITE(FORM,9200) NSCL,NBLK
 9200 FORMAT(
     .       '(1X,''             d(Sigma)/dx       No_Sample '',',
     .       '1PE9.2,',I1,'(',I2,'X,E9.2))')
             WRITE(6,9150)
 9150        FORMAT(70X,'Linear Scale indicated by "*"')
             WRITE(6,FMT=FORM) VMIN,(SCALE(L),L=1,NSCL)
             WRITE(6,9250)  CHAR1
 9250        FORMAT(1X,
     .             '+--------+--------------------+-----------+',
     .       A80 )
       ELSE
             WRITE(FORM,9205) NSCL,NBLK
 9205        FORMAT(
     .       '(1X,''           Log (dN/dx)      dN/dx    '',1PE9.2,',
     .       I1,'(',I2,'X,E9.2))')
             WRITE(6,9210) NTOTAL
 9210        FORMAT(1X,'Total number of events =',I10,
     .       37X,'Number of events in Linear scale, indicated by "*"')
             WRITE(6,FMT=FORM) VMIN,(SCALE(L),L=1,NSCL)
             WRITE(6,9251)  CHAR1
 9251        FORMAT(1X,
     .             '+--------+--------------+--------------+',
     .       A80 )
       ENDIF
 
       VX    = ABS(XMAX)
       XM    = ABS(XMIN)
       IF( XM .GT. VX ) VX = XM
 
       CALL XHORDR( VX, F2, ORD, IORD )
 
       IF( VMIN .LT. 0.0 ) THEN
           V1    = VMIN
           NUMBL = 1
           DO 150 I = 1, 80
              V2    = V1 + UNIT
              IF( V1 .LE. 0.0 .AND. V2 .GE. 0.0 ) THEN
                  NUMBL  = I
                  GO TO 180
              ENDIF
              V1    = V2
  150      CONTINUE
       ENDIF
 
  180  DO 300 I = 0,NXBIN
          VX   = VAL(I)
          IF( VMIN .GE. 0.0 ) THEN
              IF( VX .GT. 0.0 ) THEN
                  NUMBL  = (VLOG(I) - VLMIN)/UNITL + 1.0
                  NUMB   = VX/UNIT + 1.0
              ELSE
                  NUMBL  = 0
                  NUMB   = 0
              ENDIF
              IF( NUMB .GT. 80 ) NUMB = 80
              IF( NUMBL.GT. 80 ) NUMBL= 80
              DO 200 K = 1,80
                 IF(     ( K .GT. NUMBL) .AND. (K .GT. NUMB ) ) THEN
                           IF( K .EQ. 80 ) THEN
                               CHAR(K:K) = AI
                           ELSE
                               CHAR(K:K) = BLNK
                           ENDIF
                 ELSEIF( ( K .LE. NUMBL) .AND. (K .GT. NUMB )) THEN
                           CHAR(K:K) = OO
                 ELSEIF( ( K .GT. NUMBL) .AND. (K .LE. NUMB )) THEN
                           CHAR(K:K) = STAR
                 ELSEIF( ( K .LE. NUMBL) .AND. (K .LE. NUMB)) THEN
                           IF( NUMB .GE. NUMBL ) THEN
                               CHAR(K:K) = OO
                           ELSE
                               CHAR(K:K) = STAR
                           ENDIF
                 ENDIF
  200         CONTINUE
          ELSE
 
              V1          = VMIN
              NHIG        = 1
              DO 220  J = 1, 80
                 V2     = V1 + UNIT
                 IF( VX .GE. V1 .AND. VX .LT. V2 ) THEN
                     NHIG   = J
                     GO TO 240
                 ENDIF
                 V1    = V2
  220         CONTINUE
  240         NLOW   = NUMBL
              IF( NHIG .LT. NLOW) THEN
                  NX    = NHIG
                  NHIG  = NLOW
                  NLOW  = NX
              ENDIF
 
              DO 250 K = 1, 79
                 IF(     K .EQ. NUMBL ) THEN
                         CHAR(K:K) = AI
                 ELSEIF( K .GT. NHIG ) THEN
                         CHAR(K:K) = BLNK
                 ELSEIF( K .LT. NLOW ) THEN
                         CHAR(K:K) = BLNK
                 ELSE
                     IF( K .EQ. NHIG .AND. K .EQ. NLOW) THEN
                         CHAR(K:K) = AI
                     ELSE
                         CHAR(K:K) = STAR
                     ENDIF
                 ENDIF
  250         CONTINUE
              CHAR(80:80) = AI
          ENDIF
 
          IF( IFG .EQ. 0 ) THEN
 
              NX     = IBUF(I+IP2)
              VX     = VAL(I)
              VX1    = VX
              IF( VX .LT. 0.0 ) VX1 = -VX
              CALL XHORDR( VX1, F2, ORDER, IORDR )
              F2     = VX/ORDER
 
 
              IF(     I .EQ. 0 ) THEN
                      WRITE(6,9300) IORD,F2,IORDR,NX,CHAR
 9300                 FORMAT(1X,'I   E',I3,' I',F6.3,9X,'E',I3,
     .                                     ' I',I10,' I',A80)
              ELSEIF( I .EQ. NXBIN ) THEN
                      WRITE(6,9320) IORD,F2,IORDR,NX,CHAR
 9320                 FORMAT(1X,'I   E',I3,' I',F6.3,9X,'E',I3,
     .                                     ' I',I10,' I',A80)
              ELSE
                      XM    = (XMIN + DEV*(I-1))/ORD
                      VE     = VERR(I)/ORDER
                      WRITE(6,9340) XM,F2,VE,IORDR,NX,CHAR
 9340                 FORMAT(1X,'I ',F6.3,' I',F6.3,' +-',F5.3,' E',I3,
     .                                    ' I',I10,' I',A80)
             ENDIF
          ELSE
             NX  = VAL(I)
             IF(     I .EQ. 0 ) THEN
                   WRITE(6,9400) IORD,VLOG(I),NX,CHAR
 9400              FORMAT(1X,'I   E',I3,' I',G13.4,' I',
     .                                            I13,' I',A80)
             ELSEIF( I .EQ. NXBIN ) THEN
                   WRITE(6,9420) IORD,VLOG(I),NX,CHAR
 9420              FORMAT(1X,'I   E',I3,' I',G13.4,' I',
     .                                            I13,' I',A80)
             ELSE
                   XM  = (XMIN + DEV*(I - 1))/ORD
                   WRITE(6,9440) XM,VLOG(I),NX,CHAR
 9440              FORMAT(1X,'I ',F6.3,' I',G13.4,' I',
     .                                        I13,' I',A80)
             ENDIF
          ENDIF
  300  CONTINUE
 
       IF( VMIN .GE. 0.0 ) THEN
           CALL XHSCLE( 1, VLMIN, VLMAX, VLSTP, UNITL,
     .                     NSCL,  NBLK, SCALE,  CHAR1)
           MXNSC = 74/(NBLK+10)
           IF( NSCL .GT. MXNSC) NSCL = MXNSC
           VXMIN  = 10**VLMIN
       ENDIF
 
       IF( IFG .EQ. 0 ) THEN
           WRITE(6,9250) CHAR1
           IF( VMIN .GE. 0.0 ) THEN
               IF( NSCL .GT. 0 ) THEN
                   WRITE(FORM,9200) NSCL,NBLK
                   WRITE(6,FMT=FORM) VXMIN,(SCALE(L),L=1,NSCL)
               ENDIF
               WRITE(6,9260)
 9260          FORMAT(68X,'Logarithmic Scale indicated by "O"')
           ELSE
               WRITE(6,FMT=FORM) VMIN,(SCALE(L),L=1,NSCL)
           ENDIF
       ELSE
           WRITE(6,9251) CHAR1
           WRITE(FORM,9205) NSCL,NBLK
           WRITE(6,FMT=FORM) VXMIN,(SCALE(L),L=1,NSCL)
           WRITE(6,9360)
 9360      FORMAT(62X,'Number of Events in Logarithmic Scale,',
     .           ' indicated by "O"')
       ENDIF
 
C
  500  CONTINUE
 
      RETURN
      END
C***********************************************************************
C*                                                                     *
C*============================================================         *
C*  SUBROUTINE XHRNGE( IFLG, VMIN, VMAX, VTMIN, VTMAX, STEP)           *
C*============================================================         *
C*((Function))                                                         *
C*    Determine the vertical range of the histogram.                   *
C*((Input))                                                            *
C*    IFLG   : Flag which indicates whether logarithmic or linear      *
C*             scale.  IFLG = ( 1 / any other ) = ( log / linear )     *
C*    VMIN,VMAX : Minimum and maximum values of vertical window.       *
C*((Output))                                                           *
C*    VTMIN,VTMAX : Minimum and maxmum values of optimized vertical    *
C*                  window.                                            *
C*    STEP   : step of scale for the optimized vertical window         *
C*((Author))                                                           *
C*    S.Kawabata    Oct '85  at KEK                                    *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE XHRNGE( IFLG, VMIN, VMAX, VTMIN, VTMAX, STEP)
C
C     IFLG =    1 : Log scale
C            other: Linear scale
C
      PARAMETER ( NBIN  = 25 )
      REAL    WIND(NBIN),STP1(NBIN),STP2(NBIN)
C
      DATA WIND/
     .   1.00, 1.10, 1.20, 1.30, 1.40, 1.50, 1.60, 1.80, 2.00,  2.20,
     .   2.50, 2.70, 3.00, 3.30, 3.60, 4.00, 4.50, 5.00, 5.50,  6.00,
     .   6.50, 7.00, 8.00, 9.00, 10.0/
      DATA STP1/
     .   0.20, 0.22, 0.30, 0.26, 0.28, 0.30, 0.32, 0.36, 0.40,  0.44,
     .   0.50, 0.54, 0.60, 0.66, 0.60, 0.80, 0.90, 1.00, 1.10,  1.00,
     .   1.30, 1.00, 1.60, 1.80, 2.00/
C     DATA STP1/
C    .   0.20, 0.20, 0.20, 0.25, 0.25, 0.30, 0.30, 0.30, 0.50,  0.50,
C    .   0.50, 0.50, 0.50, 0.50, 0.50, 1.00, 1.00, 1.00, 1.00,  1.00,
C    .   1.00, 2.00, 2.00, 2.00, 2.00/
      DATA STP2/
     .   1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00,  1.00,
     .   1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00,  1.00,
     .   1.00, 2.00, 2.00, 2.00, 2.00/
C
          XMAX   = VMAX
          XMIN   = VMIN
          IFLAG  = IFLG
          IF( IFLG .NE. 1 .AND. VMIN .LT. 0.0 ) THEN
              IF( VMAX .LE. 0.0 )THEN
                  IFLAG  = 2
                  XMAX  = - VMIN
                  XMIN  =  0.0
              ELSE
                  AVMIN  = - VMIN
                  XMIN  =0.0
                  IF( VMAX .GE. AVMIN ) THEN
                      IFLAG  = 3
                      XMAX  = VMAX
                      XMIN1 = AVMIN
                  ELSE
                      IFLAG  = 4
                      XMAX  = AVMIN
                      XMIN1 = VMAX
                  ENDIF
              ENDIF
          ENDIF
          DSCALE = XMAX - XMIN
          CALL XHORDR( DSCALE, DSF2, DSORDR, IORD)
 
          DO 100 I = 2, 25
             IF( DSF2 .GE. WIND(I-1) .AND.
     .           DSF2 .LE. WIND( I )       ) GO TO 200
 100      CONTINUE
          I = 25
C
 200      CONTINUE
 
          XMAX = WIND(I)*DSORDR + XMIN
          IF(     DSORDR .GE. 10.0 .OR. IFLG .NE. 1 ) THEN
                  STEP1  = STP1(I)
                  STEP   = STEP1*DSORDR
          ELSE
                  STEP1  = STP2(I)
                  STEP   = STEP1
          ENDIF
 
          IF(     IFLAG .LE. 1 ) THEN
                  VTMAX  = XMAX
                  VTMIN  = XMIN
          ELSEIF( IFLAG .EQ. 2 ) THEN
                  VTMAX  = XMIN
                  VTMIN  = -XMAX
          ELSE
 
                  XPLUS   = 0.0
                  DO 300 J = 1, 10
                     XPLUS = XPLUS + STEP
                     IF( XPLUS .GT. XMIN1 ) GO TO 400
 300              CONTINUE
 400              XMIN = XPLUS
                  XMAX = XMAX
                  IF( IFIX((WIND(I)+0.1)/STEP1)+J .GT. 7 ) THEN
                      STEP = 2.0*STEP
                  ENDIF
                  IF( IFLAG .EQ. 3 ) THEN
                      VTMAX  = XMAX
                      VTMIN  = -XMIN
                  ELSE
                      VTMAX  = XMIN
                      VTMIN  = -XMAX
                  ENDIF
          ENDIF
C
      RETURN
      END
C***********************************************************************
C*==================================                                   *
C*    SUBROUTINE XHSAVE( LUNIT, ID )                                   *
C*==================================                                   *
C*((Purpose))                                                          *
C*     To write the ID-th histogram on the unit LUNIT.                 *
C*((Input))                                                            *
C*     LUNIT: Logical unit number                                      *
C*     ID   : Historgram ID                                            *
C*                                                                     *
C*((Author))                                                           *
C*      S.Kawabata   June '90 at KEK                                   *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE XHSAVE( LUNIT, ID )
C
      REAL*8         SI,SI2,SWGT,SCHI,SCALLS,ATACC,WGT
      COMMON /BASE3/ SI,SI2,SWGT,SCHI,SCALLS,ATACC,NSU,IT,WGT
 
      PARAMETER ( ILH = 50, IDH = 50 )
      INTEGER*2 XHASH,DHASH,MAXL,NHIST,MAPL,IFBASE,MAXD,NSCAT,MAPD
      COMMON/PLOTH/ XHASH(ILH,13),DHASH(IDH,14),IFBASE(ILH),
     .              MAXL, NHIST, MAPL(4,ILH),
     .              MAXD, NSCAT, MAPD(4,IDH),
     .              NW
      PARAMETER ( ISL = 281 )
      COMMON/PLOTB/ BUFF( ISL )
      INTEGER  IBUF( ISL )
      EQUIVALENCE (IBUF(1),BUFF(1))
 
C
      IF( NHIST .GT. 0 ) THEN
C
          I  = IABS(MOD( ID, 13 )) + 1
          IF( XHASH(1, I) .EQ. 1 ) THEN
            IF( ID .EQ. MAPL( 1, XHASH(2,I))) THEN
                IHIST = XHASH(2,I)
                GO TO 200
            ENDIF
          ELSEIF( XHASH(1, I) .GT. 1 ) THEN
            DO 100 K = 2, XHASH(1,I)+1
               IF( ID .EQ. MAPL( 1, XHASH(K,I))) THEN
                   IHIST = XHASH(K,I)
                   GO TO 200
               ENDIF
  100       CONTINUE
          ENDIF
      ENDIF
 
      WRITE(6,9000) ID
 9000 FORMAT(1X,'************* Warning from XHSAVE *************',
     .      /1X,' Histogram ID(',I5,' ) is illegal.',
     .      /1X,' This call is neglected.',
     .      /1X,'**********************************************')
      RETURN
 
 
 
  200 NTOTAL= SCALLS
      IP1   = MAPL(2,IHIST)
      XMIN  = BUFF(IP1)
      XMAX  = BUFF(IP1+1)
      NXBIN = IBUF(IP1+2)
      DEV   = BUFF(IP1+3)
      IP2   = MAPL(3,IHIST)
      IP3   = MAPL(4,IHIST)
 
      WRITE(6,9200) ID,LUNIT,(BUFF(I),I=IP3+1,IP3+15),
     .              NTOTAL,NXBIN,DEV
 9200 FORMAT(/1H1,
     .      1X,'** Histogram ID(',I5,' ) was saved in Unit(',I2,') **',
     .      /1X,'Title : ',15A4,
     .      /1X,'Entries     =',I10,
     .      /1X,'No. of bins =',I10,'  Width =',G13.4)
 
      WRITE( LUNIT, 9300) ID
 9300 FORMAT('(...... ID =',I5,' ......')
 
      WRITE( LUNIT, 9400)
 9400 FORMAT(1X,'NEW FRAME',
     .      /1X,'SET FONT DUPLEX')
      WRITE( LUNIT, 9500) (BUFF(I), I=IP3+1,IP3+15)
 9500 FORMAT(1X,'TITLE TOP ','''',15A4,'''')
 
      WRITE( LUNIT, 9600) XMIN,XMAX
 9600 FORMAT(1X,'SET LIMIT X FROM ',G12.4,'TO ',G12.4)
 
      WRITE( LUNIT, 9700)
 9700 FORMAT(1X,'SET INTENSITY 4',
     .      /1X,'SET TICK SIZE 0.04',
     .      /1X,'SET ORDER X Y DY')
 
      IPF   = IP2 + 156
      IPF2  = IPF + 52
      FACT       = 1./(NTOTAL*DEV)
      DO 400 I = 1, NXBIN
         TX     = BUFF(I+IPF)
         NX     = IBUF(I+IP2)
         VLS    = TX*FACT
         IF( NX .GT. 1 ) THEN
             DEV2   =  NX*BUFF(I+IPF2)-TX*TX
             IF( DEV2 .LE. 0.0 ) THEN
                 VER = 0.0
             ELSE
                 VER = FACT*SQRT( DEV2/( NX-1 ))
             ENDIF
         ELSEIF( NX .EQ. 1 ) THEN
             VER = VLS
         ELSE
             VER = 0.0
         ENDIF
         XX     = XMIN + DEV*(FLOAT(I) - 0.5)
         WRITE( LUNIT, 9800) XX, VLS, VER
 9800    FORMAT(1X,E11.4,3X,E14.7,3X,E14.7)
  400  CONTINUE
 
      WRITE( LUNIT, 9900)
C9900  FORMAT( 1X,'HIST')
9900  FORMAT( 1X,'PLOT')
 
      RETURN
      END
C
C***********************************************************************
C*                                                                     *
C*===================================================================  *
C*  SUBROUTINE XHSCLE( IFLG,VMIN,VMAX,VSTP,UNIT,NSCL,NBLK,SCALE,CHAR)  *
C*===================================================================  *
C*((Function))                                                         *
C*    Determine the vertical scale and make it's format                *
C*((Input))                                                            *
C*    IFLG   : Flag which indicates whether logarithmic or linear      *
C*             scale.  IFLG = ( 1 / any other ) = ( log / linear )     *
C*    VMIN,VMAX : Minimum and maximum values of vertical window.       *
C*    VSTEP  : Step of unit scale                                      *
C*    UNIT   : Unit of one mark *(or o)                                *
C*((Output))                                                           *
C*    NSCL   : Number of scale mark                                    *
C*    NBLK   : Number of blanks between scale marks                    *
C*    CHAR   : Format of scale                                         *
C*((Author))                                                           *
C*    S.Kawabata    Oct '85  at KEK                                    *
C*                                                                     *
C***********************************************************************
 
      SUBROUTINE XHSCLE( IFLG,VMIN,VMAX,VSTP,UNIT,NSCL,NBLK,SCALE,CHAR)
C
      REAL SCALE(*)
      REAL SCAL(0:10)
      CHARACTER*80 CHAR
      CHARACTER*1 PLUS,MINUS
      DATA PLUS /'+'/, MINUS /'-'/
 
C     IFLG =    1 : Log scale
C            other: Linear scale
 
      NBLK  = 0
      NSCL  = 0
      SCAL(NSCL) = 0.
      STV   = VSTP + VMIN
      STV1  = STV
      VAL1  = VMIN
      CHAR(80:80) = PLUS
      DO  100   I = 1, 79
          VAL2    = VAL1 + UNIT
          IF( STV .GE. VAL1 .AND. STV .LT. VAL2 ) THEN
              CHAR(I:I)  = PLUS
              NSCL       = NSCL + 1
              SCAL(NSCL) = I
              IF( IFLG .EQ. 1 ) THEN
                 SCALE(NSCL) = 10.0**STV
              ELSE
                 IF(     STV1 .EQ. 0.0 ) THEN
                         SCALE(NSCL)  = STV
                 ELSEIF( ABS(STV/STV1) .LT. 1.E-2 ) THEN
                         SCALE(NSCL)  = 0.0
                 ELSE
                         SCALE(NSCL) = STV
                 ENDIF
                 STV1       = STV
              ENDIF
              STV        = STV + VSTP
          ELSE
              CHAR(I:I) = MINUS
          ENDIF
          VAL1      = VAL2
  100 CONTINUE
C
      IF( NSCL .EQ. 0 ) THEN
          NSCL     = 1
          NBLK     = 70
          IF( IFLG .EQ. 1 ) THEN
             SCALE(1) = 10.0**VMAX
          ELSE
             SCALE(1) = VMAX
          ENDIF
      ELSE
          NBLK    = SCAL(NSCL)/NSCL - 9
      ENDIF
C
      RETURN
      END
C
C***********************************************************************
C*=================================                                    *
C* SUBROUTINE BSTIME( TIME, IFLG )                                     *
C*=================================                                    *
C*((Purpose))                                                          *
C*        Interface routine to get used CPU time from FORTRAN          *
C*        Library routine CLOCK etc.                                   *
C*((Input))                                                            *
C*        IFLG  : Flag                                                 *
C*          IFLG = 0 : Initialization of clock routine.                *
C*          IFLG = 1 : Get used CPU time.                              *
C*((Output))                                                           *
C*        TIME  : Used CPU time in second.                             *
C*                                                                     *
C*       Coded by S.Kawabata        Oct. '85                           *
C*       Modified by Hirotomo Iwasaki  for use on VAX     Apr. '88     *
C*       Modified by Hirotomo Iwasaki  for use on UNIX    Jul. '89     *
C*                                                                     *
C***********************************************************************
C
      SUBROUTINE BSTIME( TIME, IFLG )
C                                 common/BSTMVX/ on VAX only
C     COMMON/ BSTMVX / T0
C                                 real tarray(2) on UNIX only
      REAL TARRAY(2)
C
      IF( IFLG .NE. 0 ) THEN
C                                 FACOM Only
C         CALL CLOCK( TIME, 0, 1 )
C                                 HITAC Only
C         CALL CLOCK( TIME, 5 )
C                                 VAX Only 
C         TIME = SECNDS( T0 )
C                                 UNIX Only
          TDUMMY = ETIME( TARRAY )
          TIME   = TARRAY( 1 )
C
C     ELSE
C                                 HITAC Only
C         CALL CLOCK
C                                 VAX Only 
C         T0   = SECNDS( 0.0 )
C         TIME = SECNDS( T0 )
C
      ENDIF
C
      RETURN
      END

