C From:	TKYVAX::MURAYAMA     "H. Murayama, Univ. of Tokyo" 21-SEP-1990 13:31:00.17
C To:	HIROVX::IWASAKI
C CC:	
C Subj:	DHELAS.FOR used in e+e-Z
C
C  *********************************************************************
C  *********************************************************************
C  ***                                                               ***
C  ***                                                               ***
C  ***   HHH    HH   EEEEEEEEE   LL              AA        SSSSS     ***
C  ***   HHH    HH   EEEEEEEEE   LL             AAAA      SSSSSSS    ***
C  ***   HHH    HH   EE          LL            AA  AA    SS     SS   ***
C  ***   HHH    HH   EE          LL           AA    AA   SS     SS   ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS          ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS          ***
C  ***   HHHHHHHHH   EEEEEEE     LL          AAA    AA     SSS       ***
C  ***   HHHHHHHHH   EEEEEEE     LL          AAA    AA       SSS     ***
C  ***   HHH    HH   EE          LL          AAA    AA          SS   ***
C  ***   HHH    HH   EE          LL          AAAAAAAAA          SS   ***
C  ***   HHH    HH   EE          LL          AAAAAAAAA   SS     SS   ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS     SS   ***
C  ***   HHH    HH   EE          LL          AAA    AA   SS     SS   ***
C  ***   HHH    HH   EEEEEEEEE   LLLLLLLLL   AAA    AA    SSSSSSS    ***
C  ***   HHH    HH   EEEEEEEEE   LLLLLLLLL   AAA    AA     SSSSS     ***
C  ***                                                               ***
C  ***              written by H. Murayama & I. Watanabe             ***
C  ***                                                               ***
C  ***                          ver.  1.6                            ***
C  ***                       12th  July  1990                        ***
C  ***                                                               ***
C  ***   For the formalism and notations,                            ***
C  ***   see the "APPENDIX A" of the following reference:            ***
C  ***     K. Hagiwara, H. Iwasaki and D. Zeppenfeld,                ***
C  ***     "Single Weak Boson Production at TeV e+ e- Colliders"     ***
C  ***     KEK preprint, KEK-TH-255 (1990)                           ***
C  ***                                                               ***
C  *********************************************************************
C  *********************************************************************
C
C  The subroutines are named as the followings.
C
C   | f >                  : incoming massive  fermion       ==>  IMXXXX
C                          : incoming massless fermion       ==>  I0XXXX
C                          : incoming electron/positron      ==>  IEXXXX
C   < f |                  : outgoing massive  fermion       ==>  OMXXXX
C                          : outgoing massless fermion       ==>  O0XXXX
C                          : outgoing electron/positron      ==>  OEXXXX
C   epsilon^mu , epsilon*^mu  : massive  polalization vector ==>  VMXXXX
C                             : massless polarization vector ==>  V0XXXX
C   | f' V f >                : incoming fermion with vector ==>  FVIXXX
C                             : initial electron with photon ==>  EAIXXX
C   < f V f' |                : outgoing fermion with vector ==>  OVFXXX
C                             : final positron with photon   ==>  OAEXXX
C   | V f >         : truncated incoming fermion with vector ==>  VIXXXX
C   < f V |         : truncated outgoing fermion with vector ==>  OVXXXX
C   J^mu (< f' | V | f >)  : current with massive  vector    ==>  JMXXXX
C                          : current with massless vector    ==>  J0XXXX
C                          : current with photon and Z       ==>  J3XXXX
C                          : t-channel photon from e-/e+     ==>  JEEAXX
C                          : current truncated               ==>  JTXXXX
C   < f' | f >             : scalar product of fermions      ==>  SXXXXX
C   < f' | S | f >         : scalar current with propagator  ==>  SMXXXX
C                          : scalar current truncated        ==>  STXXXX
C   Gamma (WM,WP,W3)       : three-point vertex of W's & W3  ==>  WW3XXX
C   Gamma (WM,WP,WM,WP)    : four-point vertex of W's        ==>  WWWWXX
C   Gamma (WM,W3,WP,W3)    : four-point vertex of W's & W3's ==>  W3W3XX
C
C   g_mu_nu V1^mu V2^nu : dot-product of two complex vectors ==>  CDOTXX
C   g_mu_nu P1^mu P2^nu : dot-product of two real vectors    ==>  RDOTXX
C   V*^mu               : conjugation of complex vector      ==>  CONJGX
C   P1+P2               : summation of real vectors          ==>  PSUMXX
C   P1-P2               : subtraction of real vectors        ==>  PSUBXX
C   -P                  : negative of real vector            ==>  PSIGNX
C   (P^0,-P^1,-P^2,-P^3): mirror of real vector              ==>  PMIRRX
C   P(P^0,P^1,P^2,P^3)  : set up momentum                    ==>  MOMNTX
C   P_boosted           : Lorents boost of real vector       ==>  BOOSTX
C
C   set up coupling constants                                ==>  COUPLX
C
C **********************************************************************
C
      SUBROUTINE IMXXXX(P,NHEL,NSF , SI)
C
C This subroutine computes the INCOMING massive spinor wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the incoming fermion
C       integer NHEL = -1 or 1 : helicity of the fermion
C       integer NSF  = -1 or 1 : +1 for particle, -1 for anti-particle
C
C OUTPUT:
C       complex SI(4)          : spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),CHI(2)
      REAL*8    P(0:3),SF(2),SFOMEG(2)
C
      PP=SQRT(P(1)**2+P(2)**2+P(3)**2)
      NH=NHEL*NSF
      SF(1)=DBLE(1+NSF+(1-NSF)*NH)*.5D0
      SF(2)=DBLE(1+NSF-(1-NSF)*NH)*.5D0
      SFOMEG(1)=SF(1)*SQRT(DMAX1(P(0)-DSIGN(PP,DBLE(NH)),0.D0))
      SFOMEG(2)=SF(2)*SQRT(DMAX1(P(0)+DSIGN(PP,DBLE(NH)),0.D0))
      PP3=DMAX1(PP+P(3),0.D0)
      CHI(1)=DCMPLX(SQRT(PP3*.5D0/PP))
      IF (PP3.EQ.0.D0) THEN
         CHI(2)=-DCMPLX(NH)
      ELSE
         CHI(2)=DCMPLX( NH*P(1) , P(2) )/SQRT(2.D0*PP*PP3)
      END IF
      IP=(3+NH)/2
      IM=(3-NH)/2
      SI(1)=SFOMEG(1)*CHI(IM)
      SI(2)=SFOMEG(1)*CHI(IP)
      SI(3)=SFOMEG(2)*CHI(IM)
      SI(4)=SFOMEG(2)*CHI(IP)
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE I0XXXX(P,NHEL,NSF , SI)
C
C This subroutine computes the INCOMING massless spinor wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the incoming fermion
C       integer NHEL = -1 or 1 : helicity of the fermion
C       integer NSF  = -1 or 1 : +1 for particle, -1 for anti-particle
C
C OUTPUT:
C       complex SI(4)          : spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),SPIN(2)
      REAL*8    P(0:3)
C
      NH=NHEL*NSF
      SQP0P3=SQRT(DMAX1(P(0)+P(3),0.D0))*NSF
      SPIN(1)=DCMPLX(SQP0P3)
      IF (SQP0P3.EQ.0.D0) THEN
         SPIN(2)=-DCMPLX(NHEL)*SQRT(2.D0*P(0))
      ELSE
         SPIN(2)=DCMPLX( NH*P(1) , P(2) )/SQP0P3
      END IF
      IF (NH.EQ.1) THEN
        SI(1)=DCMPLX(0.D0)
        SI(2)=DCMPLX(0.D0)
        SI(3)=SPIN(1)
        SI(4)=SPIN(2)
      ELSE
        SI(1)=SPIN(2)
        SI(2)=SPIN(1)
        SI(3)=DCMPLX(0.D0)
        SI(4)=DCMPLX(0.D0)
      ENDIF
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE IEXXXX(P,NHEL,NSF , SI)
C
C This subroutine computes the INCOMING electron/positron wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the incoming fermion
C       integer NHEL = -1 or 1 : helicity of the fermion
C       integer NSF  = -1 or 1 : +1 for particle, -1 for anti-particle
C
C OUTPUT:
C       complex SI(4)          : spinor
C
      IMPLICIT REAL*8 (A-H,M,O-Z)
      COMPLEX*16 SI(4),CHI(2)
      REAL*8    P(0:3),SF(2),SFOMEG(2),OMEGA(2)
C
      ME=DBLE(0.51099906D-3)
      PP=MIN(P(0),SQRT(P(1)**2+P(2)**2+P(3)**2))
      NH=NHEL*NSF
      SF(1)=DBLE(1+NSF+(1-NSF)*NH)*.5D0
      SF(2)=DBLE(1+NSF-(1-NSF)*NH)*.5D0
      OMEGA(1)=SQRT(P(0)+PP)
      OMEGA(2)=ME/OMEGA(1)
      IP=(3+NH)/2
      IM=(3-NH)/2
      SFOMEG(1)=SF(1)*OMEGA(IP)
      SFOMEG(2)=SF(2)*OMEGA(IM)
      PP3=DMAX1(PP+P(3),0.D0)
      CHI(1)=DCMPLX(SQRT(PP3*.5D0/PP))
      IF (PP3.EQ.0.D0) THEN
         CHI(2)=-DCMPLX(NH)
      ELSE
         CHI(2)=DCMPLX( NH*P(1) , P(2) )/SQRT(2.D0*PP*PP3)
      END IF
      SI(1)=SFOMEG(1)*CHI(IM)
      SI(2)=SFOMEG(1)*CHI(IP)
      SI(3)=SFOMEG(2)*CHI(IM)
      SI(4)=SFOMEG(2)*CHI(IP)
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE OMXXXX(P,NHEL,NSF , SO)
C
C This subroutine computes the OUTGOING massive spinor wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the outgoing fermion
C       integer NHEL = -1 or 1 : helicity of the fermion
C       integer NSF  = -1 or 1 : +1 for particle, -1 for anti-particle
C
C OUTPUT:
C       complex SO(4)          : spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SO(4),CHI(2)
      REAL*8    P(0:3),SF(2),SFOMEG(2)
C
      PP=SQRT(P(1)**2+P(2)**2+P(3)**2)
      NH=NHEL*NSF
      SF(1)=DBLE(1+NSF+(1-NSF)*NH)*.5D0
      SF(2)=DBLE(1+NSF-(1-NSF)*NH)*.5D0
      SFOMEG(1)=SF(1)*SQRT(DMAX1(P(0)-DSIGN(PP,DBLE(NH)),0.D0))
      SFOMEG(2)=SF(2)*SQRT(DMAX1(P(0)+DSIGN(PP,DBLE(NH)),0.D0))
      PP3=DMAX1(PP+P(3),0.D0)
      CHI(1)=DCMPLX(SQRT(PP3*.5D0/PP))
      IF (PP3.EQ.0.D0) THEN
         CHI(2)=-DCMPLX(NH)
      ELSE
         CHI(2)=DCMPLX( NH*P(1) , -P(2) )/SQRT(2.D0*PP*PP3)
      END IF
      IP=(3+NH)/2
      IM=(3-NH)/2
      SO(1)=SFOMEG(1)*CHI(IM)
      SO(2)=SFOMEG(1)*CHI(IP)
      SO(3)=SFOMEG(2)*CHI(IM)
      SO(4)=SFOMEG(2)*CHI(IP)
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE O0XXXX(P,NHEL,NSF , SO)
C
C This subroutine computes the OUTGOING massless spinor wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the outgoing fermion
C       integer NHEL = -1 or 1 : helicity of the fermion
C       integer NSF  = -1 or 1 : +1 for particle, -1 for anti-particle
C
C OUTPUT:
C       complex SO(4)          : spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SO(4),SPIN(2)
      REAL*8    P(0:3)
C
      NH=NHEL*NSF
      SQP0P3=SQRT(DMAX1(P(0)+P(3),0.D0))*NSF
      SPIN(1)=DCMPLX(SQP0P3)
      IF (SQP0P3.EQ.0.D0) THEN
         SPIN(2)=-DCMPLX(NHEL)*SQRT(2.D0*P(0))
      ELSE
         SPIN(2)=DCMPLX( NH*P(1) , -P(2) )/SQP0P3
      END IF
      IF (NH.EQ.1) THEN
        SO(1)=DCMPLX(0.D0)
        SO(2)=DCMPLX(0.D0)
        SO(3)=SPIN(1)
        SO(4)=SPIN(2)
      ELSE
        SO(1)=SPIN(2)
        SO(2)=SPIN(1)
        SO(3)=DCMPLX(0.D0)
        SO(4)=DCMPLX(0.D0)
      ENDIF
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE OEXXXX(P,NHEL,NSF , SO)
C
C This subroutine computes the OUTGOING electron/positron wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the outgoing fermion
C       integer NHEL = -1 or 1 : helicity of the fermion
C       integer NSF  = -1 or 1 : +1 for particle, -1 for anti-particle
C
C OUTPUT:
C       complex SO(4)          : spinor
C
      IMPLICIT REAL*8 (A-H,M,O-Z)
      COMPLEX*16 SO(4),CHI(2)
      REAL*8    P(0:3),SF(2),SFOMEG(2),OMEGA(2)
C
      ME=DBLE(0.51099906D-3)
      PP=MIN(P(0),SQRT(P(1)**2+P(2)**2+P(3)**2))
      NH=NHEL*NSF
      SF(1)=DBLE(1+NSF+(1-NSF)*NH)*.5D0
      SF(2)=DBLE(1+NSF-(1-NSF)*NH)*.5D0
      OMEGA(1)=SQRT(P(0)+PP)
      OMEGA(2)=ME/OMEGA(1)
      IP=(3+NH)/2
      IM=(3-NH)/2
      SFOMEG(1)=SF(1)*OMEGA(IP)
      SFOMEG(2)=SF(2)*OMEGA(IM)
      PP3=DMAX1(PP+P(3),0.D0)
      CHI(1)=DCMPLX(SQRT(PP3*.5D0/PP))
      IF (PP3.EQ.0.D0) THEN
         CHI(2)=-DCMPLX(NH)
      ELSE
         CHI(2)=DCMPLX( NH*P(1) , -P(2) )/SQRT(2.D0*PP*PP3)
      END IF
      SO(1)=SFOMEG(1)*CHI(IM)
      SO(2)=SFOMEG(1)*CHI(IP)
      SO(3)=SFOMEG(2)*CHI(IM)
      SO(4)=SFOMEG(2)*CHI(IP)
      RETURN          
      END
C
C ======================================================================
C
      SUBROUTINE VMXXXX(P,NHEL,NSV , VECT)
C
C This subroutine computes the MASSIVE VECTOR wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the vector particle
C       integer NHEL = -1, 0, 1: helicity of the vector particle
C       integer NSV  = -1 or 1 : +1 for outgoing, -1 for incoming
C
C OUTPUT:
C       complex VECT(0:3)      : wavefunction of the vector particle
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 VECT(0:3)
      REAL*8    P(0:3)
C
      SQH=SQRT(.5D0)
      HEL=DBLE(NHEL)
      HEL0=1.D0-ABS(HEL)
      SV=DBLE(NSV)
      PP=SQRT(P(1)**2+P(2)**2+P(3)**2)
      PT=SQRT(P(1)**2+P(2)**2)
      VMASS=SQRT((P(0)-PP)*(P(0)+PP))
      EMP=P(0)/(VMASS*PP)
        VECT(0)= DCMPLX( HEL0*PP/VMASS )
        VECT(3)= DCMPLX( HEL0*P(3)*EMP+HEL*PT/PP*SQH )
      IF (PT.NE.0.D0) THEN
         PZPT=P(3)/(PP*PT)*SQH*HEL
        VECT(1)= DCMPLX( HEL0*P(1)*EMP-P(1)*PZPT ,
     &                 -ABS(HEL)*SV*P(2)/PT*SQH  )
        VECT(2)= DCMPLX( HEL0*P(2)*EMP-P(2)*PZPT ,
     &                  ABS(HEL)*SV*P(1)/PT*SQH  )
      ELSE
        VECT(1)= DCMPLX( -HEL*SQH )
        VECT(2)= DCMPLX( 0.D0 , ABS(HEL)*DSIGN(SQH,P(3))*SV )
      END IF
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE V0XXXX(P,NHEL,NSV , VECT)
C
C This subroutine computes the MASSLESS VECTOR wavefunction.
C
C INPUT:
C       real    P(0:3)         : four-momentum of the vector particle
C       integer NHEL = -1 or 1 : helicity of the vector particle
C       integer NSV  = -1 or 1 : +1 for outgoing, -1 for incoming
C
C OUTPUT:
C       complex VECT(0:3)      : wavefunction of the vector particle
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 VECT(0:3)
      REAL*8    P(0:3)
C
      SQH=SQRT(.5D0)
      HEL=DBLE(NHEL)
      SV=DBLE(NSV)
      PP=P(0)
      PT=SQRT(P(1)**2+P(2)**2)
        VECT(0)= DCMPLX( 0.D0 )
        VECT(3)= DCMPLX( HEL*PT/PP*SQH )
      IF (PT.NE.0.D0) THEN
         PZPT=P(3)/(PP*PT)*SQH*HEL
        VECT(1)= DCMPLX( -P(1)*PZPT , -SV*P(2)/PT*SQH )
        VECT(2)= DCMPLX( -P(2)*PZPT ,  SV*P(1)/PT*SQH )
      ELSE
        VECT(1)= DCMPLX( -SQH*HEL )
        VECT(2)= DCMPLX( 0.D0 , DSIGN(SQH,P(3))*SV )
      END IF
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE FVIXXX(SI,G,VECT,PF,FMASS,FWIDTH , SFVI)
C
C This subroutine computes an off-shell INCOMING spinor wavefunction
C AFTER emitting a vector particle.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       real    G(2)           : coupling constants 
C       complex VECT(0:3)      : wavefunction of the vector particle
C       real    PF(0:3)        : four-momentum of the fermion 
C                                AFTER emitting the vector paticle
C       real    FMASS          : mass of the fermion
C                                AFTER emitting the vector paticle
C       real    FWIDTH         : width of the fermion
C                                AFTER emitting the vector paticle
C
C OUTPUT:
C       complex SFVI(4)        : off-shell spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),VECT(0:3),SFVI(4),CI,SL1,SL2,SR1,SR2,D
      REAL*8    G(2),PF(0:3)
C
      CI=DCMPLX(0.D0,1.D0)
      PF2=PF(0)**2-PF(1)**2-PF(2)**2-PF(3)**2
      D=1.D0/DCMPLX( PF2-FMASS**2 , 
     &                FMASS*FWIDTH*DMAX1(DSIGN(1.D0,PF2),0.D0) )
      SL1 =  (VECT(0)+   VECT(3))*SI(1)
     &      +(VECT(1)-CI*VECT(2))*SI(2)
      SL2 =  (VECT(1)+CI*VECT(2))*SI(1)
     &      +(VECT(0)-   VECT(3))*SI(2)
      IF (G(2).EQ.0.D0) GOTO 10
      SR1 =  (VECT(0)-   VECT(3))*SI(3)
     &      -(VECT(1)-CI*VECT(2))*SI(4)
      SR2 = -(VECT(1)+CI*VECT(2))*SI(3)
     &      +(VECT(0)+   VECT(3))*SI(4)
      SFVI(1) = ( G(1)*( (PF(0)-PF(3))*SL1-(PF(1)-CI*PF(2))*SL2)
     &           +G(2)*FMASS*SR1)*D
      SFVI(2) = ( G(1)*(-(PF(1)+CI*PF(2))*SL1+(PF(0)+PF(3))*SL2)
     &           +G(2)*FMASS*SR2)*D
      SFVI(3) = ( G(2)*( (PF(0)+PF(3))*SR1+(PF(1)-CI*PF(2))*SR2)
     &           +G(1)*FMASS*SL1)*D
      SFVI(4) = ( G(2)*( (PF(1)+CI*PF(2))*SR1+(PF(0)-PF(3))*SR2)
     &           +G(1)*FMASS*SL2)*D
      RETURN          
10    SFVI(1) = G(1)*( (PF(0)-PF(3))*SL1-(PF(1)-CI*PF(2))*SL2)*D
      SFVI(2) = G(1)*(-(PF(1)+CI*PF(2))*SL1+(PF(0)+PF(3))*SL2)*D
      SFVI(3) = G(1)*FMASS*SL1*D
      SFVI(4) = G(1)*FMASS*SL2*D
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE EAIXXX(EB,EA,SHLF,CHLF,PHI,NHE,NHA , SEAI)
C
C This subroutine computes the off-shell electron wavefunction after
C emitting a photon from the electron beam with special care on the 
C small angle region.
C
C INPUT:
C       real    EB             : the energy of the beam e-
C       real    EA             : the energy of the final photon
C       real    SHLF           : sin(theta/2) of theta of the final photon
C       real    CHLF           : cos(theta/2) of theta of the final photon
C       real    PHI            : phi of the final photon
C       integer NHE  = -1 or 1 : helicity of the beam e-
C       integer NHA  = -1 or 1 : helicity of the final photon
C
C OUTPUT:
C       complex SEAI(4)        : off-shell electron wavefunction
C
      IMPLICIT REAL*8 (A-H,M,O-Z)
      COMPLEX*16 SEAI(4)
C
      ME   = DBLE(0.51099906D-3)
C      ALPHA= 1.D0/128.D0
      ALPHA= 1.D0/DBLE(137.0359895)
      GAL  = SQRT(ALPHA*4.D0*3.14159265D0)
C
      RME  = ME**2/EB**2
      X    = EA/EB
      C    = (CHLF+SHLF)*(CHLF-SHLF)
      S    = 2.D0*CHLF*SHLF
      D    =-1.D0/(2.D0*X*EB**2*(2.D0*SHLF**2+.5D0*RME*C))
      COEFF=-GAL*SQRT(EB)*DBLE(NHA*NHE)*D
C
      SEAI((5-3*NHE)/2)=-COEFF*ME*S*(1.D0+X*DBLE(1+NHA*NHE)*.5D0)
     &                  *DBLE(NHE)
      SEAI((5-NHE)/2)  = COEFF*ME*X*CHLF**2*DBLE(1+NHA*NHE)
     &                  *DCMPLX(COS(PHI),DBLE(NHE)*SIN(PHI))
      SEAI((5+NHE)/2)  = COEFF*EB*S*(-2.D0+X*DBLE(1-NHE*NHA))*DBLE(NHE)
      SEAI((5+3*NHE)/2)= COEFF*EB*X*DCMPLX(COS(PHI),DBLE(NHE)*SIN(PHI))
     &                  *DBLE(1-NHA*NHE)*2.D0*SHLF**2
C
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE OVFXXX(SO,G,VECT,PF,FMASS,FWIDTH , SOVF)
C
C This subroutine computes the off-shell OUTGOING spinor wavefunction
C BEFORE emitting the vector particle.
C
C INPUT:
C       complex SO(4)          : outgoing spinor
C       real    G(2)           : coupling constants 
C       complex VECT(0:3)      : wavefunction of the vector particle
C       real    PF(0:3)        : four-momentum of the fermion 
C                                BEFORE emitting the vector paticle
C       real    FMASS          : mass of the fermion
C                                BEFORE emitting the vector paticle
C       real    FWIDTH         : width of the fermion
C                                BEFORE emitting the vector paticle
C
C OUTPUT:
C       complex SOVF(4)        : off-shell spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SO(4),VECT(0:3),SOVF(4),CI,SL1,SL2,SR1,SR2,D
      REAL*8    G(2),PF(0:3)
C
      CI=DCMPLX(0.D0,1.D0)
      PF2=PF(0)**2-PF(1)**2-PF(2)**2-PF(3)**2
      D=1.D0/DCMPLX( PF2-FMASS**2 , 
     &                FMASS*FWIDTH*DMAX1(DSIGN(1.D0,PF2),0.D0) )
      SL1 =  (VECT(0)+   VECT(3))*SO(1)
     &      +(VECT(1)+CI*VECT(2))*SO(2)
      SL2 =  (VECT(1)-CI*VECT(2))*SO(1)
     &      +(VECT(0)-   VECT(3))*SO(2)
      IF (G(2).EQ.0.D0) GOTO 10
      SR1 =  (VECT(0)-   VECT(3))*SO(3)
     &      -(VECT(1)+CI*VECT(2))*SO(4)
      SR2 = -(VECT(1)-CI*VECT(2))*SO(3)
     &      +(VECT(0)+   VECT(3))*SO(4)
      SOVF(1) = ( G(1)*((PF(0)-PF(3))*SL1-(PF(1)+CI*PF(2))*SL2)
     &           +G(2)*FMASS*SR1)*D
      SOVF(2) = ( G(1)*(-(PF(1)-CI*PF(2))*SL1+(PF(0)+PF(3))*SL2)
     &           +G(2)*FMASS*SR2)*D
      SOVF(3) = ( G(2)*((PF(0)+PF(3))*SR1+(PF(1)+CI*PF(2))*SR2)
     &           +G(1)*FMASS*SL1)*D
      SOVF(4) = ( G(2)*((PF(1)-CI*PF(2))*SR1+(PF(0)-PF(3))*SR2)
     &           +G(1)*FMASS*SL2)*D
      RETURN          
10    SOVF(1) = G(1)*( (PF(0)-PF(3))*SL1-(PF(1)+CI*PF(2))*SL2)*D
      SOVF(2) = G(1)*(-(PF(1)-CI*PF(2))*SL1+(PF(0)+PF(3))*SL2)*D
      SOVF(3) = G(1)*FMASS*SL1*D
      SOVF(4) = G(1)*FMASS*SL2*D
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE OAEXXX(EB,EA,SHLF,CHLF,PHI,NHE,NHA , SOAE)
C
C This subroutine computes the off-shell positron wavefunction after
C emitting a photon from the positron beam with special care on the 
C small angle region.
C
C INPUT:
C       real    EB             : the energy of the beam e+
C       real    EA             : the energy of the final photon
C       real    SHLF           : sin(theta/2) of theta of the final photon
C       real    CHLF           : cos(theta/2) of theta of the final photon
C       real    PHI            : phi of the final photon
C       integer NHE  = -1 or 1 : helicity of the beam e+
C       integer NHA  = -1 or 1 : helicity of the final photon
C
C OUTPUT:
C       complex SOAE(4)        : off-shell positron wavefunction
C
      IMPLICIT REAL*8 (A-H,M,O-Z)
      COMPLEX*16 SOAE(4)
C
      ME   = DBLE(0.51099906D-3)
C      ALPHA= 1.D0/128.D0
      ALPHA= 1.D0/DBLE(137.0359895)
      GAL  = SQRT(ALPHA*4.D0*3.14159265D0)
C
      RME  = ME**2/EB**2
      X    = EA/EB
      C    = (CHLF+SHLF)*(CHLF-SHLF)
      S    = 2.D0*CHLF*SHLF
      D    =-1.D0/(2.D0*X*EB**2*(2.D0*CHLF**2-.5D0*RME*C))
      COEFF= GAL*SQRT(EB)*DBLE(NHA*NHE)*D
C
      SOAE((5-3*NHE)/2)= COEFF*EB*S*(-2.D0+X*DBLE(1-NHA*NHE))
      SOAE((5-NHE)/2)  = COEFF*EB*X*2.D0*CHLF**2*DBLE(NHA-NHE)
     &                  *DCMPLX(COS(PHI),-DBLE(NHE)*SIN(PHI))
      SOAE((5+NHE)/2)  = COEFF*ME*S*(1.D0+X*DBLE(1+NHA*NHE)*.5D0)
      SOAE((5+NHE*3)/2)= COEFF*ME*X*SHLF**2*DBLE(1+NHA*NHE)
     &                  *DBLE(NHE)*DCMPLX(COS(PHI),-DBLE(NHE)*SIN(PHI))
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE VIXXXX(SI,G,VECT , SVI)
C
C This subroutine computes a truncated off-shell INCOMING spinor 
C wavefunction AFTER emitting a vector particle.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       real    G(2)           : coupling constants 
C       complex VECT(0:3)      : wavefunction of the vector particle
C
C OUTPUT:
C       complex SVI(4)         : off-shell spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),VECT(0:3),SVI(4),CI
      REAL*8    G(2)
C
      CI=DCMPLX(0.D0,1.D0)
      IF (G(2).EQ.0.D0) THEN
        SVI(1) = DCMPLX(0.D0)
        SVI(2) = DCMPLX(0.D0)
      ELSE
        SVI(1) =( (VECT(0)-   VECT(3))*SI(3)
     &           -(VECT(1)-CI*VECT(2))*SI(4))*G(2)
        SVI(2) =(-(VECT(1)+CI*VECT(2))*SI(3)
     &           +(VECT(0)+   VECT(3))*SI(4))*G(2)
      ENDIF
        SVI(3) =( (VECT(0)+   VECT(3))*SI(1)
     &           +(VECT(1)-CI*VECT(2))*SI(2))*G(1)
        SVI(4) =( (VECT(1)+CI*VECT(2))*SI(1)
     &           +(VECT(0)-   VECT(3))*SI(2))*G(1)
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE OVXXXX(SO,G,VECT , SOV)
C
C This subroutine computes a truncated off-shell OUTGOING spinor 
C wavefunction BEFORE emitting the vector particle.
C
C INPUT:
C       complex SO(4)          : outgoing spinor
C       real    G(2)           : coupling constants 
C       complex VECT(0:3)      : wavefunction of the vector particle
C
C OUTPUT:
C       complex SOV(4)         : off-shell spinor
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SO(4),VECT(0:3),SOV(4),CI
      REAL*8    G(2)
C
      CI=DCMPLX(0.D0,1.D0)
      IF (G(2).EQ.0.D0) THEN
        SOV(1) = DCMPLX(0.D0)
        SOV(2) = DCMPLX(0.D0)
      ELSE
        SOV(1) = ( (VECT(0)-   VECT(3))*SO(3)
     &            -(VECT(1)+CI*VECT(2))*SO(4))*G(2)
        SOV(2) = (-(VECT(1)-CI*VECT(2))*SO(3)
     &            +(VECT(0)+   VECT(3))*SO(4))*G(2)
      ENDIF
        SOV(3) = ( (VECT(0)+   VECT(3))*SO(1)
     &            +(VECT(1)+CI*VECT(2))*SO(2))*G(1)
        SOV(4) = ( (VECT(1)-CI*VECT(2))*SO(1)
     &            +(VECT(0)-   VECT(3))*SO(2))*G(1)
      RETURN          
      END
C
C ======================================================================
C
      SUBROUTINE JMXXXX(SI,G,Q,VMASS,VWIDTH,SO , CURRNT)
C
C This subroutine computes a bispinor current that couples to a massive 
C vector particle including to the VECTOR PROPAGATOR in the UNITARY 
C GAUGE.
C If the vector particle is massless, use the subroutine J0XXXX.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       complex SO(4)          : outgoing spinor
C       real    G(2)           : coupling constants 
C       real    Q(0:3)         : momentum transfer
C       real    VMASS          : mass of the vector particle
C       real    VWIDTH         : width of the vector particle
C
C OUTPUT:
C       complex CURRNT(0:3)    : vector and axial current of the fermion
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),SO(4),CURRNT(0:3),C0,C1,C2,C3,CS,D
      REAL*8    G(2),Q(0:3)
C
      Q2=Q(0)**2-Q(1)**2-Q(2)**2-Q(3)**2
      D=-1.D0/DCMPLX( Q2-VMASS**2 , 
     &                 VMASS*VWIDTH*DMAX1(DSIGN(1.D0,Q2),0.D0) )
C
C  For the running width, use below instead of the above D.
C      D=-1.D0/DCMPLX( Q2-VMASS**2 , DMAX1(0.D0,VWIDTH*Q2/VMASS) )
C
      IF (G(2).EQ.0.D0) GOTO 10
      C0 =  G(1)*( SO(1)*SI(1)+SO(2)*SI(2))
     &     +G(2)*( SO(3)*SI(3)+SO(4)*SI(4))
      C1 = -G(1)*( SO(1)*SI(2)+SO(2)*SI(1))
     &     +G(2)*( SO(3)*SI(4)+SO(4)*SI(3))
      C2 =( G(1)*( SO(1)*SI(2)-SO(2)*SI(1)) 
     &     +G(2)*(-SO(3)*SI(4)+SO(4)*SI(3)))*DCMPLX(0.D0,1.D0)
      C3 =  G(1)*(-SO(1)*SI(1)+SO(2)*SI(2))
     &     +G(2)*( SO(3)*SI(3)-SO(4)*SI(4))
      CS=(Q(0)*C0-Q(1)*C1-Q(2)*C2-Q(3)*C3)/VMASS**2
      CURRNT(0)= (C0-CS*Q(0))*D
      CURRNT(1)= (C1-CS*Q(1))*D
      CURRNT(2)= (C2-CS*Q(2))*D
      CURRNT(3)= (C3-CS*Q(3))*D
      RETURN
10    D=D*G(1)
      C0 =  SO(1)*SI(1)+SO(2)*SI(2)
      C1 = -SO(1)*SI(2)-SO(2)*SI(1)
      C2 =( SO(1)*SI(2)-SO(2)*SI(1))*DCMPLX(0.D0,1.D0)
      C3 = -SO(1)*SI(1)+SO(2)*SI(2)
      CS=(Q(0)*C0-Q(1)*C1-Q(2)*C2-Q(3)*C3)/VMASS**2
      CURRNT(0)= (C0-CS*Q(0))*D
      CURRNT(1)= (C1-CS*Q(1))*D
      CURRNT(2)= (C2-CS*Q(2))*D
      CURRNT(3)= (C3-CS*Q(3))*D
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE J0XXXX(SI,G,Q,SO , CURRNT)
C
C This subroutine computes a bispinor current that couples to a massless 
C vector particle including to the VECTOR PROPAGATOR in the FEYNMAN 
C GAUGE.
C If the vector particle is massive, use the subroutine JMXXXX.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       complex SO(4)          : outgoing spinor
C       real    G(2)           : coupling constants 
C       real    Q(0:3)         : momentum transfer
C
C OUTPUT:
C       complex CURRNT(0:3)    : vector and axial current of the fermion
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),SO(4),CURRNT(0:3)
      REAL*8    G(2),Q(0:3)
C
      D=-1.D0/(Q(0)**2-Q(1)**2-Q(2)**2-Q(3)**2)
      IF (G(2).EQ.0.D0) GOTO 10
      CURRNT(0) =( G(1)*( SO(1)*SI(1)+SO(2)*SI(2))
     &            +G(2)*( SO(3)*SI(3)+SO(4)*SI(4)) )*D
      CURRNT(1) =(-G(1)*( SO(1)*SI(2)+SO(2)*SI(1))
     &            +G(2)*( SO(3)*SI(4)+SO(4)*SI(3)) )*D
      CURRNT(2) =( G(1)*( SO(1)*SI(2)-SO(2)*SI(1))
     &            +G(2)*(-SO(3)*SI(4)+SO(4)*SI(3)) )*D*DCMPLX(0.,1.)
      CURRNT(3) =( G(1)*(-SO(1)*SI(1)+SO(2)*SI(2))
     &            +G(2)*( SO(3)*SI(3)-SO(4)*SI(4)) )*D
      RETURN
10    D=D*G(1)
      CURRNT(0) =  ( SO(1)*SI(1)+SO(2)*SI(2))*D
      CURRNT(1) = -( SO(1)*SI(2)+SO(2)*SI(1))*D
      CURRNT(2) =  ( SO(1)*SI(2)-SO(2)*SI(1))*D*DCMPLX(0.,1.)
      CURRNT(3) =  (-SO(1)*SI(1)+SO(2)*SI(2))*D
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE J3XXXX(SI,GZF,GAF,Q,ZMASS,ZWIDTH,SO , CURRNT)
C
C This subroutine computes the sum of the photon current J0XXXX and
C Z current JMXXXX with the suitable weight for the input of WW3XXX
C or W3W3XX. We use the photon propagator in the Feynman gauge, and
C Z propagator in the UNITARY GAUGE.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       complex SO(4)          : outgoing spinor
C       real    GZF(2)         : coupling constant with Z of the fermion
C       real    GAF(2)         : coupling constant with A of the fermion
C       real    Q(0:3)         : momentum transfer
C       real    ZMASS          : mass of the Z boson
C       real    ZWIDTH         : width of the Z boson
C
C OUTPUT:
C       complex CURRNT(0:3)    : vector and axial current of the fermion
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),SO(4),CURRNT(0:3),
     &        C0L,C1L,C2L,C3L,CSL,C0R,C1R,C2R,C3R,CSR,DZ,DDIF
      REAL*8    GZF(2),GAF(2),Q(0:3)
C
      Q2=Q(0)**2-Q(1)**2-Q(2)**2-Q(3)**2
      DA=-1.D0/Q2
      DZ=-1.D0/DCMPLX( Q2-ZMASS**2 , 
     &                  ZMASS*ZWIDTH*DMAX1(DSIGN(1.D0,Q2),0.D0) )
C
C  For the running width, use below instead of the above DZ.
C       DZ=-1.D0/DCMPLX( Q2-ZMASS**2 , DMAX1(0.D0,ZWIDTH*Q2/ZMASS) )
C
      DDIF = DCMPLX(ZMASS**2 , 
     &             -ZMASS*ZWIDTH*DMAX1(DSIGN(1.D0,Q2),0.D0) )*DA*DZ
C DDIF is the difference : DDIF=DA-DZ
      CW = 1.D0/SQRT(1.D0+(GZF(2)/GAF(2))**2)
      SW = SQRT((1.D0-CW)*(1.D0+CW))
      GN = GAF(2)*SW
C GN is the symbol for Q*SW where Q is the charge of the particle.
      GZ3L = GZF(1)*CW
      GA3L = GAF(1)*SW
      C0L =   SO(1)*SI(1)+SO(2)*SI(2)
      C0R =   SO(3)*SI(3)+SO(4)*SI(4)
      C1L = -(SO(1)*SI(2)+SO(2)*SI(1))
      C1R =   SO(3)*SI(4)+SO(4)*SI(3)
      C2L =  (SO(1)*SI(2)-SO(2)*SI(1))*DCMPLX(0.,1.)
      C2R = (-SO(3)*SI(4)+SO(4)*SI(3))*DCMPLX(0.,1.)
      C3L =  -SO(1)*SI(1)+SO(2)*SI(2)
      C3R =   SO(3)*SI(3)-SO(4)*SI(4)
      CSL = (Q(0)*C0L-Q(1)*C1L-Q(2)*C2L-Q(3)*C3L)/ZMASS**2
      CSR = (Q(0)*C0R-Q(1)*C1R-Q(2)*C2R-Q(3)*C3R)/ZMASS**2
      CURRNT(0)=  GZ3L*DZ*(C0L-CSL*Q(0))+GA3L*C0L*DA
     &          + GN*(C0R*DDIF-CSR*Q(0)*DZ)
      CURRNT(1)=  GZ3L*DZ*(C1L-CSL*Q(1))+GA3L*C1L*DA
     &          + GN*(C1R*DDIF-CSR*Q(1)*DZ)
      CURRNT(2)=  GZ3L*DZ*(C2L-CSL*Q(2))+GA3L*C2L*DA
     &          + GN*(C2R*DDIF-CSR*Q(2)*DZ)
      CURRNT(3)=  GZ3L*DZ*(C3L-CSL*Q(3))+GA3L*C3L*DA
     &          + GN*(C3R*DDIF-CSR*Q(3)*DZ)
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE JEEAXX(EB,EF,SHLF,CHLF,PHI,NHB,NHF,NSF , CURRNT)
C
C This subroutine computes the off-shell photon wavefunction emitted 
C from the electron or positron beam with special care on the small
C angle region.
C
C INPUT:
C       real    EB             : the energy of the beam e-/e+
C       real    EF             : the energy of the final e-/e+
C       real    SHLF           : sin(theta/2) of theta of the final e-/e+
C       real    CHLF           : cos(theta/2) of theta of the final e-/e+
C       real    PHI            : phi of the final e-/e+
C       integer NHB  = -1 or 1 : helicity of the beam e-/e+
C       integer NHF  = -1 or 1 : helicity of the final e-/e+
C       integer NSF  = -1 or 1 : +1 for electron, -1 for positron
C
C OUTPUT:
C       complex CURRNT(0:3)    : off-shell photon wavefunction
C
      IMPLICIT REAL*8 (A-H,M,O-Z)
      COMPLEX*16 CURRNT(0:3),COEFF
      REAL*8    CS(2)
C
      ME   = DBLE(0.51099906D-3)
C      ALPHA= 1.D0/128.D0
      ALPHA= 1.D0/DBLE(137.0359895)
      GAL  = SQRT(ALPHA*4.D0*3.14159265D0)
C
      HI = DBLE(NHB)
      SF = DBLE(NSF)
      CS((3+NSF)/2) = SHLF
      CS((3-NSF)/2) = CHLF
C CS(1)=CHLF and CS(2)=SHLF for electron
C CS(1)=SHLF and CS(2)=CHLF for positron
      X  = EF/EB
      RME= ME**2/EB**2
      Q2 =-EB**2*(4.D0*(X-RME)*CS(2)**2
     &    +RME*(1.D0-X)**2/X*(CS(1)+CS(2))*(CS(1)-CS(2)))
C
      IF (NHB.EQ.NHF) THEN
         COEFF  = GAL*2.D0*EB*SQRT(X)*CS(2)/Q2
     &           *(DBLE(1+NSF)-DBLE(1-NSF)
     &             *DCMPLX(COS(PHI),-SIN(PHI)*HI))*.5D0
         CURRNT(0)= 0.D0
         CURRNT(1)= COEFF*(DCMPLX(COS(PHI),-SF*HI*SIN(PHI))
     &             +2.D0*X/(1.D0-X)*CS(1)**2*COS(PHI))
         CURRNT(2)= COEFF*(DCMPLX(SIN(PHI),SF*HI*COS(PHI))
     &             +2.D0*X/(1.D0-X)*CS(1)**2*SIN(PHI))
         CURRNT(3)= COEFF*(-SF*2.D0*X/(1.D0-X)*CS(1)*CS(2))
      ELSE
         COEFF  = GAL*ME/Q2/SQRT(X)
     &           *(DBLE(1+NSF)+DBLE(1-NSF)
     &             *DCMPLX(COS(PHI),SIN(PHI)*HI))*.5D0
         CURRNT(0)=-COEFF*HI*(1.D0+X)*CS(2)
     &              *DCMPLX(COS(PHI),SF*HI*SIN(PHI))
         CURRNT(1)= COEFF*HI*(1.D0-X)*CS(1)
         CURRNT(2)= CURRNT(1)*DCMPLX(0.D0,SF*HI)
         CURRNT(3)= CURRNT(0)*SF*(1.D0-X)/(1.D0+X)
      END IF
      RETURN          
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE JTXXXX(SI,G,SO , CURRNT)
C
C This subroutine computes the vector and axial vector coupling of the
C fermion with vector particle WITHOUT the vector propagator.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       complex SO(4)          : outgoing spinor
C       real    G(2)           : coupling constants 
C
C OUTPUT:
C       complex CURRNT(0:3)    : vector and axial current of the fermion
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),SO(4),CURRNT(0:3)
      REAL*8    G(2)
C
      IF (G(2).EQ.0.D0) GOTO 10
      CURRNT(0) =  G(1)*( SO(1)*SI(1)+SO(2)*SI(2))
     &            +G(2)*( SO(3)*SI(3)+SO(4)*SI(4))
      CURRNT(1) = -G(1)*( SO(1)*SI(2)+SO(2)*SI(1))
     &            +G(2)*( SO(3)*SI(4)+SO(4)*SI(3))
      CURRNT(2) =( G(1)*( SO(1)*SI(2)-SO(2)*SI(1))
     &            +G(2)*(-SO(3)*SI(4)+SO(4)*SI(3)))*DCMPLX(0.,1.)
      CURRNT(3) = -G(1)*( SO(1)*SI(1)-SO(2)*SI(2))
     &            +G(2)*( SO(3)*SI(3)-SO(4)*SI(4))
      RETURN
10    CURRNT(0) =  G(1)*( SO(1)*SI(1)+SO(2)*SI(2))
      CURRNT(1) = -G(1)*( SO(1)*SI(2)+SO(2)*SI(1))
      CURRNT(2) =  G(1)*( SO(1)*SI(2)-SO(2)*SI(1))*DCMPLX(0.,1.)
      CURRNT(3) =  G(1)*(-SO(1)*SI(1)+SO(2)*SI(2))
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE SXXXXX(SI,SO , SCALAR)
C
C This subroutine computes the scalar product of two spinors.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       complex SO(4)          : outgoing spinor
C
C OUTPUT:
C       complex SCALAR         : scalar product of the spinors
C
      COMPLEX*16 SI(4),SO(4),SCALAR
C
      SCALAR = SO(3)*SI(1)+SO(4)*SI(2)+SO(1)*SI(3)+SO(2)*SI(4)
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE SMXXXX(SI,G,Q,SMASS,SWIDTH,SO , SCALAR)
C
C This subroutine computes a bispinor current that couples to a scalar 
C particle including the SCALAR PROPAGATOR.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       complex SO(4)          : outgoing spinor
C       real    G(2)           : coupling constants 
C       real    Q(0:3)         : momentum transfer
C       real    SMASS          : mass of the scalar particle
C       real    SWIDTH         : width of the scalar particle
C
C OUTPUT:
C       complex SCALAR         : scalar product of the spinors
C
C The Feynman rule is
C                          ___             ___          
C                -i ( G(1) F_R  F_L + G(2) F_L F_R )   .
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),SO(4),SCALAR,D
      REAL*8    G(2),Q(0:3)
C
      Q2=Q(0)**2-Q(1)**2-Q(2)**2-Q(3)**2
      D=1.D0/DCMPLX( Q2-SMASS**2 , 
     &                SMASS*SWIDTH*DMAX1(DSIGN(1.0D0,Q2),0.0D0) )
      SCALAR = ( G(1)*(SO(3)*SI(1)+SO(4)*SI(2))
     &          +G(2)*(SO(1)*SI(3)+SO(2)*SI(4)) )*D
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE STXXXX(SI,G,SO , SCALAR)
C
C This subroutine computes a bispinor current that couples to a scalar 
C particle WITHOUT the scalar propagator.
C
C INPUT:
C       complex SI(4)          : incoming spinor
C       complex SO(4)          : outgoing spinor
C       real    G(2)           : coupling constants 
C
C OUTPUT:
C       complex SCALAR         : scalar product of the spinors
C
C The Feynman rule is
C                          ___             ___          
C                -i ( G(1) F_R  F_L + G(2) F_L F_R )   .
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 SI(4),SO(4),SCALAR
      REAL*8    G(2)
C
      SCALAR =  G(1)*(SO(3)*SI(1)+SO(4)*SI(2))
     &         +G(2)*(SO(1)*SI(3)+SO(2)*SI(4))
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE WW3XXX(WM,PWM,WP,PWP,W3,PW3,G , VERTEX)
C
C This subroutine computes the three-point coupling of the W bosons
C WITHOUT the vector propagator.
C
C INPUT:
C       complex WM(0:3)        : wavefunction VMXXXX or current JMXXXX 
C                                of W-.
C       complex WP(0:3)        : wavefunction VMXXXX or current JMXXXX 
C                                of W+.
C       complex W3(0:3)        : current J3XXXX of W3, VMXXXX of Z 
C                                or V0XXXX of A.
C       real    PWM(0:3)       : OUTGOING four-momentum of W-
C       real    PWP(0:3)       : OUTGOING four-momentum of W+
C       real    PW3(0:3)       : OUTGOING four-momentum of W3/Z/A
C       real    G              : coupling constant
C                                (GW for W3=J3, GWWZ for W3=Z, 
C                                               GWWA for W3=A.)
C
C OUTPUT:
C       complex VERTEX         : three-point function of three vectors
C
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 WM(0:3),WP(0:3),W3(0:3),VERTEX, 
     &        XV1,XV2,XV3,V12,V23,V31,P12,P13,P21,P23,P31,P32,
     &        PV1,PV2,PV3
      REAL*8    PWM(0:3),PWP(0:3),PW3(0:3)
C
      V12 =  WM(0)*WP(0)-WM(1)*WP(1)
     &      -WM(2)*WP(2)-WM(3)*WP(3)
      V23 =  WP(0)*W3(0)-WP(1)*W3(1)
     &      -WP(2)*W3(2)-WP(3)*W3(3)
      V31 =  W3(0)*WM(0)-W3(1)*WM(1)
     &      -W3(2)*WM(2)-W3(3)*WM(3)
      XV1 = 0.D0
      XV2 = 0.D0
      XV3 = 0.D0
      IF(ABS(WM(0)).GT.0.D0) THEN
        IF (ABS(WM(0)).GE.MAX(ABS(WM(1)),ABS(WM(2)),ABS(WM(3)))*1.D-1)
     &      XV1=PWM(0)/WM(0)
      ENDIF
      IF(ABS(WP(0)).GT.0.D0) THEN
        IF (ABS(WP(0)).GE.MAX(ABS(WP(1)),ABS(WP(2)),ABS(WP(3)))*1.D-1)
     &      XV2=PWP(0)/WP(0)
      ENDIF
      IF(ABS(W3(0)).GT.0.D0) THEN
        IF (ABS(W3(0)).GE.MAX(ABS(W3(1)),ABS(W3(2)),ABS(W3(3)))*1.D-1)
     &      XV3=PW3(0)/W3(0)
      ENDIF
      P12 =  (PWM(0)-XV1*WM(0))*WP(0)
     &      -(PWM(1)-XV1*WM(1))*WP(1)
     &      -(PWM(2)-XV1*WM(2))*WP(2)
     &      -(PWM(3)-XV1*WM(3))*WP(3)
      P13 =  (PWM(0)-XV1*WM(0))*W3(0)
     &      -(PWM(1)-XV1*WM(1))*W3(1)
     &      -(PWM(2)-XV1*WM(2))*W3(2)
     &      -(PWM(3)-XV1*WM(3))*W3(3)
      P21 =  (PWP(0)-XV2*WP(0))*WM(0)
     &      -(PWP(1)-XV2*WP(1))*WM(1)
     &      -(PWP(2)-XV2*WP(2))*WM(2)
     &      -(PWP(3)-XV2*WP(3))*WM(3)
      P23 =  (PWP(0)-XV2*WP(0))*W3(0)
     &      -(PWP(1)-XV2*WP(1))*W3(1)
     &      -(PWP(2)-XV2*WP(2))*W3(2)
     &      -(PWP(3)-XV2*WP(3))*W3(3)
      P31 =  (PW3(0)-XV3*W3(0))*WM(0)
     &      -(PW3(1)-XV3*W3(1))*WM(1)
     &      -(PW3(2)-XV3*W3(2))*WM(2)
     &      -(PW3(3)-XV3*W3(3))*WM(3)
      P32 =  (PW3(0)-XV3*W3(0))*WP(0)
     &      -(PW3(1)-XV3*W3(1))*WP(1)
     &      -(PW3(2)-XV3*W3(2))*WP(2)
     &      -(PW3(3)-XV3*W3(3))*WP(3)
      VERTEX = (V12*(P13-P23)+V23*(P21-P31)+V31*(P32-P12))*G
      RETURN
10    PV1 =  (PWP(0)-PW3(0))*WM(0)-(PWP(1)-PW3(1))*WM(1)
     &      -(PWP(2)-PW3(2))*WM(2)-(PWP(3)-PW3(3))*WM(3)
      PV2 =  (PW3(0)-PWM(0))*WP(0)-(PW3(1)-PWM(1))*WP(1)
     &      -(PW3(2)-PWM(2))*WP(2)-(PW3(3)-PWM(3))*WP(3)
      PV3 =  (PWM(0)-PWP(0))*W3(0)-(PWM(1)-PWP(1))*W3(1)
     &      -(PWM(2)-PWP(2))*W3(2)-(PWM(3)-PWP(3))*W3(3)
      VERTEX = (V12*PV3+V23*PV1+V31*PV2)*G
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE WWWWXX(WM1,PWM1,WP1,PWP1,WM2,PWM2,WP2,PWP2,GW,SW2,
     &                  ZMASS,ZWIDTH , VERTEX)
C
C This subroutine computes the four-point coupling of the W-/W+ bosons
C WITHOUT the external W propagators.
C
C INPUT:
C       complex WM1(0:3)       : wavefunction VMXXXX or current JMXXXX 
C                                of the OUTGOING W-.
C       complex WP1(0:3)       : wavefunction VMXXXX or current JMXXXX 
C                                of the OUTGOING W+.
C       complex WM2(0:3)       : wavefunction VMXXXX or current JMXXXX 
C                                of the OUTGOING W-.
C       complex WP2(0:3)       : wavefunction VMXXXX or current JMXXXX 
C                                of the OUTGOING W+.
C       real    PWM1(0:3)      : OUTGOING four-momentum of W-
C       real    PWP1(0:3)      : OUTGOING four-momentum of W+
C       real    PWM2(0:3)      : OUTGOING four-momentum of W-
C       real    PWP2(0:3)      : OUTGOING four-momentum of W+
C       real    GW             : weak coupling constant
C       real    SW2            : sin(theta_W)**2
C       real    ZMASS          : mass of the Z boson
C       real    ZWIDTH         : width of the Z boson
C
C OUTPUT:
C       complex VERTEX         : four-point function of three vectors
C
      IMPLICIT REAL*8 (A-H,O-U,W-Z)
      IMPLICIT COMPLEX*16 (J,V)
      COMPLEX*16    WM1(0:3),WP1(0:3),WM2(0:3),WP2(0:3),VERTEX
      COMPLEX*16 DV1(0:3),DV2(0:3),DV3(0:3),DV4(0:3),
     &           J12(0:3),J34(0:3),J14(0:3),J32(0:3),
     &           SV1,SV2,SV3,SV4,TV1,TV2,TV3,TV4,DZS,DZT,DVERTX
      REAL*8       PWM1(0:3),PWP1(0:3),PWM2(0:3),PWP2(0:3),
     &           GW,SW2,ZMASS,ZWIDTH
      REAL*8     Q(0:3),K(0:3),DP1(0:3),DP2(0:3),DP3(0:3),DP4(0:3)
C
      DV1(0) = DCMPLX(WM1(0))
      DV1(1) = DCMPLX(WM1(1))
      DV1(2) = DCMPLX(WM1(2))
      DV1(3) = DCMPLX(WM1(3))
      DP1(0) = DBLE(PWM1(0))
      DP1(1) = DBLE(PWM1(1))
      DP1(2) = DBLE(PWM1(2))
      DP1(3) = DBLE(PWM1(3))
      DV2(0) = DCMPLX(WP1(0))
      DV2(1) = DCMPLX(WP1(1))
      DV2(2) = DCMPLX(WP1(2))
      DV2(3) = DCMPLX(WP1(3))
      DP2(0) = DBLE(PWP1(0))
      DP2(1) = DBLE(PWP1(1))
      DP2(2) = DBLE(PWP1(2))
      DP2(3) = DBLE(PWP1(3))
      DV3(0) = DCMPLX(WM2(0))
      DV3(1) = DCMPLX(WM2(1))
      DV3(2) = DCMPLX(WM2(2))
      DV3(3) = DCMPLX(WM2(3))
      DP3(0) = DBLE(PWM2(0))
      DP3(1) = DBLE(PWM2(1))
      DP3(2) = DBLE(PWM2(2))
      DP3(3) = DBLE(PWM2(3))
      DV4(0) = DCMPLX(WP2(0))
      DV4(1) = DCMPLX(WP2(1))
      DV4(2) = DCMPLX(WP2(2))
      DV4(3) = DCMPLX(WP2(3))
      DP4(0) = DBLE(PWP2(0))
      DP4(1) = DBLE(PWP2(1))
      DP4(2) = DBLE(PWP2(2))
      DP4(3) = DBLE(PWP2(3))
      DMZ    = DBLE(ZMASS)
      DWIDTH = DBLE(ZWIDTH)
      DSW2   = DBLE(SW2)
C
      V12 =  DV1(0)*DV2(0)-DV1(1)*DV2(1)
     &      -DV1(2)*DV2(2)-DV1(3)*DV2(3)
      V13 =  DV1(0)*DV3(0)-DV1(1)*DV3(1)
     &      -DV1(2)*DV3(2)-DV1(3)*DV3(3)
      V14 =  DV1(0)*DV4(0)-DV1(1)*DV4(1)
     &      -DV1(2)*DV4(2)-DV1(3)*DV4(3)
      V23 =  DV2(0)*DV3(0)-DV2(1)*DV3(1)
     &      -DV2(2)*DV3(2)-DV2(3)*DV3(3)
      V24 =  DV2(0)*DV4(0)-DV2(1)*DV4(1)
     &      -DV2(2)*DV4(2)-DV2(3)*DV4(3)
      V34 =  DV3(0)*DV4(0)-DV3(1)*DV4(1)
     &      -DV3(2)*DV4(2)-DV3(3)*DV4(3)
C
      Q(0) = DP1(0) + DP2(0)
      Q(1) = DP1(1) + DP2(1)
      Q(2) = DP1(2) + DP2(2)
      Q(3) = DP1(3) + DP2(3)
      K(0) = DP1(0) + DP4(0)
      K(1) = DP1(1) + DP4(1)
      K(2) = DP1(2) + DP4(2)
      K(3) = DP1(3) + DP4(3)
C
      S = Q(0)**2-Q(1)**2-Q(2)**2-Q(3)**2
      T = K(0)**2-K(1)**2-K(2)**2-K(3)**2
C
      DAS = -1.D0/S
      DAT = -1.D0/T
      DZS = -1.D0/DCMPLX(S-DMZ**2, DMZ*DWIDTH*DMAX1(DSIGN(1.D0,S),0.D0))
      DZT = -1.D0/DCMPLX(T-DMZ**2, DMZ*DWIDTH*DMAX1(DSIGN(1.D0,T),0.D0))
C
      SV1 = (DP2(0)+Q(0))*DV1(0) -(DP2(1)+Q(1))*DV1(1)
     &     -(DP2(2)+Q(2))*DV1(2) -(DP2(3)+Q(3))*DV1(3)
      SV2 =-(DP1(0)+Q(0))*DV2(0) +(DP1(1)+Q(1))*DV2(1)
     &     +(DP1(2)+Q(2))*DV2(2) +(DP1(3)+Q(3))*DV2(3)
      SV3 = (DP4(0)-Q(0))*DV3(0) -(DP4(1)-Q(1))*DV3(1)
     &     -(DP4(2)-Q(2))*DV3(2) -(DP4(3)-Q(3))*DV3(3)
      SV4 =-(DP3(0)-Q(0))*DV4(0) +(DP3(1)-Q(1))*DV4(1)
     &     +(DP3(2)-Q(2))*DV4(2) +(DP3(3)-Q(3))*DV4(3)
C
      TV1 = (DP4(0)+K(0))*DV1(0) -(DP4(1)+K(1))*DV1(1)
     &     -(DP4(2)+K(2))*DV1(2) -(DP4(3)+K(3))*DV1(3)
      TV2 =-(DP3(0)-K(0))*DV2(0) +(DP3(1)-K(1))*DV2(1)
     &     +(DP3(2)-K(2))*DV2(2) +(DP3(3)-K(3))*DV2(3)
      TV3 = (DP2(0)-K(0))*DV3(0) -(DP2(1)-K(1))*DV3(1)
     &     -(DP2(2)-K(2))*DV3(2) -(DP2(3)-K(3))*DV3(3)
      TV4 =-(DP1(0)+K(0))*DV4(0) +(DP1(1)+K(1))*DV4(1)
     &     +(DP1(2)+K(2))*DV4(2) +(DP1(3)+K(3))*DV4(3)
C
      J12(0) =(DP1(0)-DP2(0))*V12 +SV1*DV2(0) +SV2*DV1(0)
      J12(1) =(DP1(1)-DP2(1))*V12 +SV1*DV2(1) +SV2*DV1(1)
      J12(2) =(DP1(2)-DP2(2))*V12 +SV1*DV2(2) +SV2*DV1(2)
      J12(3) =(DP1(3)-DP2(3))*V12 +SV1*DV2(3) +SV2*DV1(3)
      J34(0) =(DP3(0)-DP4(0))*V34 +SV3*DV4(0) +SV4*DV3(0)
      J34(1) =(DP3(1)-DP4(1))*V34 +SV3*DV4(1) +SV4*DV3(1)
      J34(2) =(DP3(2)-DP4(2))*V34 +SV3*DV4(2) +SV4*DV3(2)
      J34(3) =(DP3(3)-DP4(3))*V34 +SV3*DV4(3) +SV4*DV3(3)
C
      J14(0) =(DP1(0)-DP4(0))*V14 +TV1*DV4(0) +TV4*DV1(0)
      J14(1) =(DP1(1)-DP4(1))*V14 +TV1*DV4(1) +TV4*DV1(1)
      J14(2) =(DP1(2)-DP4(2))*V14 +TV1*DV4(2) +TV4*DV1(2)
      J14(3) =(DP1(3)-DP4(3))*V14 +TV1*DV4(3) +TV4*DV1(3)
      J32(0) =(DP3(0)-DP2(0))*V23 +TV3*DV2(0) +TV2*DV3(0)
      J32(1) =(DP3(1)-DP2(1))*V23 +TV3*DV2(1) +TV2*DV3(1)
      J32(2) =(DP3(2)-DP2(2))*V23 +TV3*DV2(2) +TV2*DV3(2)
      J32(3) =(DP3(3)-DP2(3))*V23 +TV3*DV2(3) +TV2*DV3(3)
C
      JS12   = Q(0)*J12(0) -Q(1)*J12(1) -Q(2)*J12(2) -Q(3)*J12(3)
      JS34   = Q(0)*J34(0) -Q(1)*J34(1) -Q(2)*J34(2) -Q(3)*J34(3)
      JS14   = K(0)*J14(0) -K(1)*J14(1) -K(2)*J14(2) -K(3)*J14(3)
      JS32   = K(0)*J32(0) -K(1)*J32(1) -K(2)*J32(2) -K(3)*J32(3)
C
      JS  = J12(0)*J34(0) -J12(1)*J34(1) -J12(2)*J34(2) -J12(3)*J34(3)
      JT  = J14(0)*J32(0) -J14(1)*J32(1) -J14(2)*J32(2) -J14(3)*J32(3)
C
      CW2    = 1.D0 -SW2
      DVERTX = V12*V34 +V14*V23 -2.D0*V13*V24
     &       +(DZS*CW2+DAS*SW2)*JS -DZS*CW2*JS12*JS34/DMZ**2
     &       +(DZT*CW2+DAT*SW2)*JT -DZT*CW2*JS14*JS32/DMZ**2
C
      VERTEX = DCMPLX(DVERTX)*GW**2
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE W3W3XX(WM,PWM,W31,PW31,WP,PWP,W32,PW32,GW31,GW32,
     &                  WMASS,WWIDTH , VERTEX)
C
C This subroutine computes the four-point coupling of the W-/W+ and 
C W3/Z/A bosons WITHOUT the external vector propagators.
C
C INPUT:
C       complex WM(0:3)        : wavefunction VMXXXX or current JMXXXX 
C                                of OUTGOING W-.
C       complex W31(0:3)       : wavefunction VMXXXX of Z, V0XXXX of A 
C                                or current J3XXXX of W3.
C       complex WP(0:3)        : wavefunction VMXXXX or current JMXXXX 
C                                of OUTGOING W+.
C       complex W32(0:3)       : wavefunction VMXXXX of Z, V0XXXX of A 
C                                or current J3XXXX of W3.
C       real    PWM(0:3)       : OUTGOING four-momentum of W-
C       real    PW31(0:3)      : OUTGOING four-momentum of W3/Z/A
C       real    PWP(0:3)       : OUTGOING four-momentum of W+
C       real    PW32(0:3)      : OUTGOING four-momentum of W3/Z/A
C       real    GW31           : coupling of W31 with W-/W+
C       real    GW32           : coupling of W32 with W-/W+
C                                (GW for W3?=J3, GWWZ for W3?=Z,
C                                                GWWA for W3?=A.)
C       real    WMASS          : mass of the W boson
C       real    WWIDTH         : width of the W boson
C
C OUTPUT:
C       complex VERTEX         : four-point function of three vectors
C
      IMPLICIT REAL*8 (A-H,O-U,W-Z)
      IMPLICIT COMPLEX*16 (J,V)
      COMPLEX*16    WM(0:3),W31(0:3),WP(0:3),W32(0:3),VERTEX
      COMPLEX*16 DV1(0:3),DV2(0:3),DV3(0:3),DV4(0:3),
     &           J12(0:3),J34(0:3),J14(0:3),J32(0:3),
     &           SV1,SV2,SV3,SV4,TV1,TV2,TV3,TV4,DWS,DWT,DVERTX
      REAL*8       PWM(0:3),PW31(0:3),PWP(0:3),PW32(0:3),
     &           GW31,GW32,WMASS,WWIDTH
      REAL*8     Q(0:3),K(0:3),DP1(0:3),DP2(0:3),DP3(0:3),DP4(0:3)
C
      DV1(0) = DCMPLX(WM(0))
      DV1(1) = DCMPLX(WM(1))
      DV1(2) = DCMPLX(WM(2))
      DV1(3) = DCMPLX(WM(3))
      DP1(0) = DBLE(PWM(0))
      DP1(1) = DBLE(PWM(1))
      DP1(2) = DBLE(PWM(2))
      DP1(3) = DBLE(PWM(3))
      DV2(0) = DCMPLX(W31(0))
      DV2(1) = DCMPLX(W31(1))
      DV2(2) = DCMPLX(W31(2))
      DV2(3) = DCMPLX(W31(3))
      DP2(0) = DBLE(PW31(0))
      DP2(1) = DBLE(PW31(1))
      DP2(2) = DBLE(PW31(2))
      DP2(3) = DBLE(PW31(3))
      DV3(0) = DCMPLX(WP(0))
      DV3(1) = DCMPLX(WP(1))
      DV3(2) = DCMPLX(WP(2))
      DV3(3) = DCMPLX(WP(3))
      DP3(0) = DBLE(PWP(0))
      DP3(1) = DBLE(PWP(1))
      DP3(2) = DBLE(PWP(2))
      DP3(3) = DBLE(PWP(3))
      DV4(0) = DCMPLX(W32(0))
      DV4(1) = DCMPLX(W32(1))
      DV4(2) = DCMPLX(W32(2))
      DV4(3) = DCMPLX(W32(3))
      DP4(0) = DBLE(PW32(0))
      DP4(1) = DBLE(PW32(1))
      DP4(2) = DBLE(PW32(2))
      DP4(3) = DBLE(PW32(3))
      DMW    = DBLE(WMASS)
      DWIDTH= DBLE(WWIDTH)
C
      V12 =  DV1(0)*DV2(0)-DV1(1)*DV2(1)
     &      -DV1(2)*DV2(2)-DV1(3)*DV2(3)
      V13 =  DV1(0)*DV3(0)-DV1(1)*DV3(1)
     &      -DV1(2)*DV3(2)-DV1(3)*DV3(3)
      V14 =  DV1(0)*DV4(0)-DV1(1)*DV4(1)
     &      -DV1(2)*DV4(2)-DV1(3)*DV4(3)
      V23 =  DV2(0)*DV3(0)-DV2(1)*DV3(1)
     &      -DV2(2)*DV3(2)-DV2(3)*DV3(3)
      V24 =  DV2(0)*DV4(0)-DV2(1)*DV4(1)
     &      -DV2(2)*DV4(2)-DV2(3)*DV4(3)
      V34 =  DV3(0)*DV4(0)-DV3(1)*DV4(1)
     &      -DV3(2)*DV4(2)-DV3(3)*DV4(3)
C
      Q(0) = DP1(0) + DP2(0)
      Q(1) = DP1(1) + DP2(1)
      Q(2) = DP1(2) + DP2(2)
      Q(3) = DP1(3) + DP2(3)
      K(0) = DP1(0) + DP4(0)
      K(1) = DP1(1) + DP4(1)
      K(2) = DP1(2) + DP4(2)
      K(3) = DP1(3) + DP4(3)
C
      S = Q(0)**2-Q(1)**2-Q(2)**2-Q(3)**2
      T = K(0)**2-K(1)**2-K(2)**2-K(3)**2
      DWS = -1.D0/DCMPLX(S-DMW**2, DMW*DWIDTH*DMAX1(DSIGN(1.D0,S),0.D0))
      DWT = -1.D0/DCMPLX(T-DMW**2, DMW*DWIDTH*DMAX1(DSIGN(1.D0,T),0.D0))
C
      SV1 = (DP2(0)+Q(0))*DV1(0) -(DP2(1)+Q(1))*DV1(1)
     &     -(DP2(2)+Q(2))*DV1(2) -(DP2(3)+Q(3))*DV1(3)
      SV2 =-(DP1(0)+Q(0))*DV2(0) +(DP1(1)+Q(1))*DV2(1)
     &     +(DP1(2)+Q(2))*DV2(2) +(DP1(3)+Q(3))*DV2(3)
      SV3 = (DP4(0)-Q(0))*DV3(0) -(DP4(1)-Q(1))*DV3(1)
     &     -(DP4(2)-Q(2))*DV3(2) -(DP4(3)-Q(3))*DV3(3)
      SV4 =-(DP3(0)-Q(0))*DV4(0) +(DP3(1)-Q(1))*DV4(1)
     &     +(DP3(2)-Q(2))*DV4(2) +(DP3(3)-Q(3))*DV4(3)
C
      TV1 = (DP4(0)+K(0))*DV1(0) -(DP4(1)+K(1))*DV1(1)
     &     -(DP4(2)+K(2))*DV1(2) -(DP4(3)+K(3))*DV1(3)
      TV2 =-(DP3(0)-K(0))*DV2(0) +(DP3(1)-K(1))*DV2(1)
     &     +(DP3(2)-K(2))*DV2(2) +(DP3(3)-K(3))*DV2(3)
      TV3 = (DP2(0)-K(0))*DV3(0) -(DP2(1)-K(1))*DV3(1)
     &     -(DP2(2)-K(2))*DV3(2) -(DP2(3)-K(3))*DV3(3)
      TV4 =-(DP1(0)+K(0))*DV4(0) +(DP1(1)+K(1))*DV4(1)
     &     +(DP1(2)+K(2))*DV4(2) +(DP1(3)+K(3))*DV4(3)
C
      J12(0) =(DP1(0)-DP2(0))*V12 +SV1*DV2(0) +SV2*DV1(0)
      J12(1) =(DP1(1)-DP2(1))*V12 +SV1*DV2(1) +SV2*DV1(1)
      J12(2) =(DP1(2)-DP2(2))*V12 +SV1*DV2(2) +SV2*DV1(2)
      J12(3) =(DP1(3)-DP2(3))*V12 +SV1*DV2(3) +SV2*DV1(3)
      J34(0) =(DP3(0)-DP4(0))*V34 +SV3*DV4(0) +SV4*DV3(0)
      J34(1) =(DP3(1)-DP4(1))*V34 +SV3*DV4(1) +SV4*DV3(1)
      J34(2) =(DP3(2)-DP4(2))*V34 +SV3*DV4(2) +SV4*DV3(2)
      J34(3) =(DP3(3)-DP4(3))*V34 +SV3*DV4(3) +SV4*DV3(3)
C
      J14(0) =(DP1(0)-DP4(0))*V14 +TV1*DV4(0) +TV4*DV1(0)
      J14(1) =(DP1(1)-DP4(1))*V14 +TV1*DV4(1) +TV4*DV1(1)
      J14(2) =(DP1(2)-DP4(2))*V14 +TV1*DV4(2) +TV4*DV1(2)
      J14(3) =(DP1(3)-DP4(3))*V14 +TV1*DV4(3) +TV4*DV1(3)
      J32(0) =(DP3(0)-DP2(0))*V23 +TV3*DV2(0) +TV2*DV3(0)
      J32(1) =(DP3(1)-DP2(1))*V23 +TV3*DV2(1) +TV2*DV3(1)
      J32(2) =(DP3(2)-DP2(2))*V23 +TV3*DV2(2) +TV2*DV3(2)
      J32(3) =(DP3(3)-DP2(3))*V23 +TV3*DV2(3) +TV2*DV3(3)
C
      JS12   = Q(0)*J12(0) -Q(1)*J12(1) -Q(2)*J12(2) -Q(3)*J12(3)
      JS34   = Q(0)*J34(0) -Q(1)*J34(1) -Q(2)*J34(2) -Q(3)*J34(3)
      JS14   = K(0)*J14(0) -K(1)*J14(1) -K(2)*J14(2) -K(3)*J14(3)
      JS32   = K(0)*J32(0) -K(1)*J32(1) -K(2)*J32(2) -K(3)*J32(3)
C
      JS  = J12(0)*J34(0) -J12(1)*J34(1) -J12(2)*J34(2) -J12(3)*J34(3)
      JT  = J14(0)*J32(0) -J14(1)*J32(1) -J14(2)*J32(2) -J14(3)*J32(3)
C
      DVERTX = V12*V34 +V14*V23 -2.D0*V13*V24
     &        +DWS*(JS -JS12*JS34/DMW**2)
     &        +DWT*(JT -JS14*JS32/DMW**2)
C
      VERTEX = -DCMPLX(DVERTX)*(GW31*GW32)
      RETURN
      END
C
C **********************************************************************
C
      SUBROUTINE CDOTXX(V1,V2 , SCALAR)
C
C This subroutine computes the scalar product of two COMPLEX vectors.
C For the real vectors (momenta), use RDOTXX subroutine.
C
C INPUT:
C       complex V1(0:3)        : vector
C       complex V2(0:3)        : vector
C
C OUTPUT:
C       complex SCALAR         : scalar product of two vectors
C
      COMPLEX*16 V1(0:3),V2(0:3),SCALAR
C
      SCALAR =  V1(0)*V2(0)-V1(1)*V2(1)
     &         -V1(2)*V2(2)-V1(3)*V2(3)
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE RDOTXX(P1,P2 , SCALAR)
C
C This subroutine computes the scalar product of two REAL vectors.
C To get an inner-product of two momenta, call this subroutine.
C For the complex vectors, use CDOTXX subroutine.
C
C INPUT:
C       real    P1(0:3)        : vector
C       real    P2(0:3)        : vector
C
C OUTPUT:
C       real    SCALAR         : scalar product of two vectors
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 P1(0:3),P2(0:3)
C
      SCALAR =  P1(0)*P2(0)-P1(1)*P2(1)
     &         -P1(2)*P2(2)-P1(3)*P2(3)
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE CONJGX(V , VCONJG)
C
C This subroutine computes the conjugation of a complex vector.
C
C INPUT:
C       complex V(0:3)         : complex vector
C
C OUTPUT:
C       complex VCONJG(0:3)    : conjugation V*
C
      COMPLEX*16 V(0:3),VCONJG(0:3)
C
      VCONJG(0)=CONJG(V(0))
      VCONJG(1)=CONJG(V(1))
      VCONJG(2)=CONJG(V(2))
      VCONJG(3)=CONJG(V(3))
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE PSUMXX(P1,P2 , PSUM)
C
C This subroutine computes the sum of two real vectors.
C
C INPUT:
C       real    P1(0:3)        : real vector
C       real    P2(0:3)        : real vector
C
C OUTPUT:
C       real    PSUM(0:3)      : summation P1+P2
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8    P1(0:3),P2(0:3),PSUM(0:3)
C
      PSUM(0)=P1(0)+P2(0)
      PSUM(1)=P1(1)+P2(1)
      PSUM(2)=P1(2)+P2(2)
      PSUM(3)=P1(3)+P2(3)
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE PSUBXX(P1,P2 , PSUB)
C
C This subroutine computes the subtraction of two real vectors.
C
C INPUT:
C       real    P1(0:3)        : real vector
C       real    P2(0:3)        : real vector
C
C OUTPUT:
C       real    PSUB(0:3)      : subtraction P1-P2
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8    P1(0:3),P2(0:3),PSUB(0:3)
C
      PSUB(0)=P1(0)-P2(0)
      PSUB(1)=P1(1)-P2(1)
      PSUB(2)=P1(2)-P2(2)
      PSUB(3)=P1(3)-P2(3)
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE PSIGNX(P , PSIGN)
C
C This subroutine reverses the sign of a four-momentum.
C
C INPUT:
C       real    P(0:3)         : real vector
C
C OUTPUT:
C       real    PSIGN(0:3)     : reversed four-vector -P
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8    P(0:3),PSIGN(0:3)
C
      PSIGN(0)=-P(0)
      PSIGN(1)=-P(1)
      PSIGN(2)=-P(2)
      PSIGN(3)=-P(3)
      RETURN
      END
C
C ----------------------------------------------------------------------
C
      SUBROUTINE PMIRRX(P , PMIRR)
C
C This subroutine computes the real vector whose signs of the spacial 
C components are reversed from the ones of the original vector.
C
C INPUT:
C       real    P(0:3)         : real vector
C
C OUTPUT:
C       real    PMIRR(0:3)     : mirror vector
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8    P(0:3),PMIRR(0:3)
C
      PMIRR(0)= P(0)
      PMIRR(1)=-P(1)
      PMIRR(2)=-P(2)
      PMIRR(3)=-P(3)
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE MOMNTX(ENERGY,MASS,COSTH,PHI , P)
C
C This subroutine sets up the momentum.
C
C INPUT:
C       real    ENERGY         : energy
C       real    MASS           : mass
C       real    COSTH          : cos(theta)
C       real    PHI            : azimuthal angle
C
C OUTPUT:
C       real    P(0:3)         : momentum
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8    MASS,P(0:3)
C
      P(0)=ENERGY
      IF (ENERGY.EQ.MASS) THEN
         P(1)=0.D0
         P(2)=0.D0
         P(3)=0.D0
      ELSE
         PP=SQRT((ENERGY-MASS)*(ENERGY+MASS))
         SINTH=SQRT((1.D0-COSTH)*(1.D0+COSTH))
         P(3)=PP*COSTH
         IF (PHI.EQ.0.D0) THEN
            P(1)=PP*SINTH
            P(2)=0.D0
         ELSE
            P(1)=PP*SINTH*COS(PHI)
            P(2)=PP*SINTH*SIN(PHI)
         END IF
      END IF
      RETURN
      END
C
C ======================================================================
C
      SUBROUTINE BOOSTX(P,Q , PBOOST)
C
C This subroutine computes the Lorentz boost of the real vector. 
C The vector P is assumed to be given in the rest frame of Q, which must 
C be a timelike vector.  PBOOST is the vector P boosted to the frame in 
C which Q is given.
C
C INPUT:
C       real    P(0:3)         : real vector which will be boosted
C       real    Q(0:3)         : vector of new origin in the old 
C                                coordinates
C
C OUTPUT:
C       real    PBOOST(0:3)    : real vector boosted
C
      IMPLICIT REAL*8 (A-H,L,M,O-Z)
      REAL*8    P(0:3),Q(0:3),PBOOST(0:3)
C
      PQ=P(1)*Q(1)+P(2)*Q(2)+P(3)*Q(3)
      QQ=Q(1)**2+Q(2)**2+Q(3)**2
C
      M=SQRT(Q(0)**2-QQ)
      LF=((Q(0)-M)*PQ/QQ+P(0))/M
      PBOOST(0)=(P(0)*Q(0)+PQ)/M
      PBOOST(1)=P(1)+Q(1)*LF
      PBOOST(2)=P(2)+Q(2)*LF
      PBOOST(3)=P(3)+Q(3)*LF
      RETURN
      END
C
C **********************************************************************
C
      SUBROUTINE COUPLX(SW2 , GAL,GAU,GAD,GWF,GZN,GZL,GZU,GZD,G1,
     &                        GW,GWWZ,GWWA)
C
C This subroutine sets up the coupling constants in the STANDARD MODEL.
C The array of couplings specifies the chirality of the INCOMING 
C fermion. G?(1) denotes a left-handed coupling, and G?(2) a right-
C handed coupling.
C
C INPUT:
C       real    SW2            : square of sine of Weinberg angle
C
C OUTPUT:
C       real    GAL(2)         : coupling with A of charged leptons
C       real    GAU(2)         : coupling with A of up-type quarks
C       real    GAD(2)         : coupling with A of down-type quarks
C       real    GWF(2)         : coupling with W-,W+ of fermions
C       real    GZN(2)         : coupling with Z of neutrinos
C       real    GZL(2)         : coupling with Z of charged leptons
C       real    GZU(2)         : coupling with Z of up-type quarks
C       real    GZD(2)         : coupling with Z of down-type quarks
C       real    G1(2)          : unit coupling of fermions
C       real    GW             : weak coupling constant
C       real    GWWZ           : coupling of W-,W+,Z
C       real    GWWA           : coupling of W-,W+,A
C 
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 GAL(2),GAU(2),GAD(2),GWF(2),
     &       GZN(2),GZL(2),GZU(2),GZD(2),G1(2)
C
C      ALPHA  = 1.D0/128.D0
      ALPHA  = 1.D0/DBLE(137.0359895)
      FOURPI = DBLE(4.*3.14159265358979323846)
      E =SQRT(ALPHA *FOURPI)
      SW=SQRT(SW2)
      CW=SQRT(1.D0-SW2)
      EZ=E/(SW*CW)
C
      GAL(1)=-E
      GAL(2)=-E
      GAU(1)= E*2.D0/3.D0
      GAU(2)= E*2.D0/3.D0
      GAD(1)=-E/3.D0
      GAD(2)=-E/3.D0
      GWF(1)= E/SQRT(2.D0*SW2)
      GWF(2)= 0.D0
      GZN(1)= EZ*.5D0
      GZN(2)= 0.D0
      GZL(1)= EZ*(-.5D0+SW2)
      GZL(2)= EZ*SW2
      GZU(1)= EZ*(.5D0-2.D0/3.D0*SW2)
      GZU(2)=-EZ*2.D0/3.D0*SW2
      GZD(1)= EZ*(-.5D0+1.D0/3.D0*SW2)
      GZD(2)= EZ/3.D0*SW2
      G1(1) = 1.D0
      G1(2) = 1.D0
C
      GW    = E/SW
      GWWZ  = E/SW*CW
      GWWA  = E
      RETURN
      END
